
/* Borland Delphi 1.0 grammar 03.VIII.1997 Robert Zierer */

%{
(*
 *
 *)
{$D+,L+,Y+}
unit Parser;{$B-}

interface
uses LexLib, YaccLib;

procedure Run_Parse;

%}

%start compilation_file

/* Lexical stuff: */

/*  ***  F O R   P A S C A L  ***  */

/* Note the Pascal keyword tokens are stropped with leading underscore
   (e.g. _AND) in the Turbo Pascal Yacc version, because these identifiers
   will be declared as token numbers in the output file generated by Yacc,
   and hence must not collide with Turbo Pascal keywords. */

/* Pascal 'reserved words' */

%token   _AND_  _AS_  _ASM_  _ARRAY_
%token   _BEGIN_  _CASE_  _CLASS_  _CONST_
%token   _CONSTRUCTOR_  _DESTRUCTOR_  _DIV_  _DO_
%token   _DOWNTO_  _ELSE_  _END_  _EXCEPT_
%token   _EXPORTS_  _FILE_  _FINALLY_  _FOR_
%token   _FUNCTION_  _GOTO_  _IF_  _IMPLEMENTATION_
%token   _IN_  _INHERITED_  _INLINE_  _INITIALIZATION_
%token   _INTERFACE_  _IS_  _LABEL_  _LIBRARY_
%token   _MOD_  _NIL_  _NOT_  _OBJECT_
%token   _OF_  _OR_  _PACKED_  _PROCEDURE_
%token   _PROGRAM_  _PROPERTY_  _RAISE_  _RECORD_
%token   _REPEAT_  _SET_  _SHL_  _SHR_
%token   _STRING_  _THEN_  _TO_  _TRY_
%token   _TYPE_  _UNIT_  _UNTIL_  _USES_
%token   _VAR_  _WHILE_  _WITH_  _XOR_

/* Pascal 'Standard-Directives' */

%token   _ABSOLUTE_  _ABSTRACT_  _ASSEMBLER_  _AT_
%token   _CDECL_  _DEFAULT_  _DYNAMIC_  _EXPORT_
%token   _EXTERNAL_  _FAR_  _FORWARD_  _INDEX_
%token   _INTERRUPT_  _MESSAGE_  _NAME_  _NEAR_
%token   _NODEFAULT_  _ON_  _OVERRIDE_  _PRIVATE_
%token   _PROTECTED_  _PUBLIC_  _PUBLISHED_  _READ_
%token   _RESIDENT_  _STORED_  _VIRTUAL_  _WRITE_

/* Pascal 'Miscellaneous' */

%token   _ASSIGNMENT_  DOTDOT  GE  LE
%token   NOTEQUAL  STARSTAR  IDENTIFIER  IMMCHAR_ID
%token   COLON
%token   COMMA  DOT  EQUAL  GT
%token   LBRAC  LPAREN  LT  MINUS
%token   PLUS  RBRAC  RPAREN  SEMICOLON
%token   SLASH  STAR  UPARROW  APE_SYMBOL
%token   ILLEGAL
%token   ET
%token   ASM_LABEL  ASM_SEPERATOR  ASM_REGISTER  ASM_OPCODE
%token   ASM_DIRECTIVE

/* right EQUAL  NOTEQUAL  LT  GT  LE  GE  _IN_    */
/* right PLUS  MINUS  _OR_  _XOR_   */
/* right STAR  SLASH  _DIV_  _MOD_  _SHL_  _SHR_  _AND_  */
/* right UMINUS  APE_SYMBOL _NOT_     */

%right _THEN_ _ELSE_            /* resolve dangling else */

%token  <Longint> UNSIGNED_INTEGER
%token  <Extended> UNSIGNED_REAL
%token  <PString> STRING_CONST

%%

/* Rules: general */

       
/***********************************************************************/
/* ist das File ein Programm(*.dpr -> *.exe), eine Unit(*.pas -> *.dcu)
   oder eine Library(*.dpr,*.pas -> *.dll) ? */
compilation_file
        : program_file {writeln('erkannt:program_file');}
        | unit_file {}
        | library_file {}
        ;

/***********************************************************************/
/** Librarys */
library_file
        : library_heading {} main_uses_clause {}
           library_block {} DOT {}
        ;

library_heading
        : _LIBRARY_ {} IDENTIFIER {} SEMICOLON {}
        ;

library_block
        : library_impl_decl_sect_list {} compound_stmt {}
        ;

library_impl_decl_sect_list
        : /* empty */ {}
        | library_impl_decl_sect_list {} library_impl_decl_sect {}
        ;

library_impl_decl_sect
        : label_decl_sect {}
        | const_decl_sect {}
        | type_decl_sect {}
        | var_decl_sect {}
        | proc_decl {}
        | func_decl {}
        | constructor_decl {}
        | destructor_decl {}
        | export_clause {}
        ;

export_clause
        : _EXPORTS_ {} exports_list {} SEMICOLON {}
        ;

exports_list
        : exports_entry {}
        | exports_list {} COMMA {} exports_entry {}
        ;

exports_entry
        : IDENTIFIER {} exports_index {} exports_name {} exports_resident {}
        ;

exports_index
        : /* empty */ {} 
        | _INDEX_ {} integer_const {}
        ;

exports_name
        : /* empty */ {}
        | _NAME_ {} string_const_2 {}
        ;

exports_resident
        : /* empty */ {}
        | _RESIDENT_
        ;

/***********************************************************************/
/** Programs */
program_file
        : program_heading {} main_uses_clause {}
           program_block {} DOT {}
        ;

/*** der Programmkopf */
/* e.g. PROGRAM test( input, output ); */
program_heading
        : /* empty */ {}
        | _PROGRAM_ {} program_name {} program_heading_2 {writeln('erk: prog ');}
        ;

program_heading_2
        : SEMICOLON {}
        | LPAREN {} program_param_list {} RPAREN {} SEMICOLON {}
        ;

program_name
        : IDENTIFIER {}
        ;

program_param_list
        : program_param {} 
        | program_param_list {} COMMA {} program_param {}
        ;

program_param
        : IDENTIFIER {}
        ;

program_block
        : program_decl_sect_list {} compound_stmt {}
        ;

program_decl_sect_list : impl_decl_sect_list {} ;

/*** die Uses-Anweisung */
/* e.g. uses winprocs, wintypes, exception, outline; */
uses_clause
        : /* empty */ {}
        | _USES_ {} used_units_list {} SEMICOLON {}
        ;

/**** Liste der verwendeten Units */
used_units_list
        : used_units_list {} COMMA {} used_unit_name {}
        | used_unit_name {}
        ;

used_unit_name : IDENTIFIER {} ;

/* e.g. uses winprocs, wintypes, exception in 'source\errorhandling', outline; */
main_uses_clause
        : /* empty */ {}
        | _USES_ {} main_used_units_list {} SEMICOLON {}
        ;

main_used_units_list
        : main_used_units_list {} COMMA {} main_used_unit_name {}
        | main_used_unit_name {}
        ;

main_used_unit_name
        : IDENTIFIER {}
        | IDENTIFIER  _IN_ {} STRING_CONST {}
        ;

/***********************************************************************/
/** Units */
unit_file
        : unit_heading {}
           interface_part {}
           implementation_part {}
           initialization_part {}
           DOT {}
        ;

unit_heading
        : _UNIT_ {} unit_name {} SEMICOLON {}
        ;

unit_name
        : IDENTIFIER {}
        ;

interface_part
        : _INTERFACE_ {} uses_clause {} int_decl_sect_list {}
        ;

implementation_part
        : _IMPLEMENTATION_ {} uses_clause {} impl_decl_sect_list {}
        ;

initialization_part
        : compound_stmt {}
        | _INITIALIZATION_ {} stmt_list {} _END_ {}
 /*      | _FINALIZATION_ {} stmt_list {} SEMICOLON 
            _INITIALIZATION_ {} stmt_list {} _END_ {} */
        | _END_ {}
        ;

/***********************************************************************/
/* Declaration sections: */

impl_decl_sect_list
        : /* empty */ {}
        | impl_decl_sect_list {} impl_decl_sect {}
        ;

impl_decl_sect
        : label_decl_sect {}
        | const_decl_sect {}
        | type_decl_sect {}
        | var_decl_sect {}
        | proc_decl {}
        | func_decl {}
        | constructor_decl {}
        | destructor_decl {}
        ;

label_decl_sect
        : _LABEL_ {} label_list {} SEMICOLON {}
        ;

label_list
        : label {}
        | label_list {} COMMA {} label {}
        ;

label   : UNSIGNED_INTEGER {}
            /* must be decimal integer in the range 0..9999 */
        |  IDENTIFIER {}
        ;

const_decl_sect
        : _CONST_ {} const_decl {}
        | const_decl_sect {} const_decl {}
        ;

type_decl_sect
        : _TYPE_ {} type_decl {}
        | type_decl_sect {} type_decl {}
        ;

var_decl_sect
        : _VAR_ {} var_decl {}
        | var_decl_sect var_decl
        ;

/***********************************************************************/
/* Interface declaration sections: */

/* These appear instead of normal declaration sections in the interface
   part of a unit. The difference is that there are no label declarations
   and for procedures and functions only the headings are given. */

int_decl_sect_list
        : /* empty */ {}
        | int_decl_sect_list {} int_decl_sect {}
        ;

int_decl_sect
        : const_decl_sect {}
        | type_decl_sect {}
        | var_decl_sect {}
        | proc_heading {}
        | func_heading {}
        ;

/***********************************************************************/
/* Constant declarations: */

const_decl
        : const_name {} EQUAL {} const {} SEMICOLON {}
        | const_name {} COLON {} type {} EQUAL {} typed_const {} SEMICOLON {}
        ;

const_name : IDENTIFIER {} ;

/* ********************************************************************* */
/* Constant Expressions and calculated Constants: */

const   : const_simple_expr {}
        | const_simple_expr  const_relop {} const_simple_expr {}
        ;


const_relop
        : EQUAL {} | NOTEQUAL {} | LT {} | GT {} | LE {} | GE {} | _IN_ {}
        ;

const_simple_expr
        : const_term {}
        | const_simple_expr  const_addop {} const_term {}
        ;

const_addop
        : PLUS {} | MINUS {} | _OR_ {} | _XOR_ {}
        ;

const_term
        : const_factor {}
        | const_term {} const_mulop {} const_factor {}
        ;

const_mulop
        : STAR {} | SLASH {} | _DIV_ {} | _MOD_ {}
        | _SHL_ {} | _SHR_ {} | _AND_ {}
        ;

const_factor
        : const_variable {} 
        | unsigned_number {}
        | string {}
        | _NIL_ {}
        | APE_SYMBOL {} const_factor {}
        | LPAREN  const RPAREN {} 		/* ist das auch noch eine Enumeration ??? */
        | _NOT_ {} const_factor {}
        | sign {} const_factor {}
        | set_const {}
       ;

const_variable             
        : IDENTIFIER {}     /*const_name; auch type oder func_call moeglich hier; qualified name */
        | const_variable {} const_variable_2 {}
        ;

const_variable_2
        : LBRAC {} const_expr_list {} RBRAC {}  /* array component */
        | DOT {} IDENTIFIER {}         /* record field; also: QualifiedName, func_call */
        | UPARROW {}  			       /* pointer value */
        | LPAREN {} const_func_expr_list {} RPAREN {}
           /* TypeCast */
           /*oder Function-Call */
           /*oder von proc_call in stmt  */
           /*also procedural type TypeCast; or function delivering a procedural type */
        ;

const_expr_list
        : const {}
        | const_expr_list {} COMMA {} const {}
        ;

const_func_expr_list
        : const {}          /* bei Abs   Ord   Chr    Pred    Hi
                                   High	 Round Length SizeOf  Lo   Succ
                                   Low   Swap  Odd    Trunc */
        | const {} COMMA {} const {}	/* nur bei Ptr */
        ;

const_elem_list
        : /* empty */ {}
        | const_elem_list1 {}
        ;

const_elem_list1
        : const_elem {}
        | const_elem_list1 {} COMMA {} const_elem {}
        ;

const_elem
        : const {}
        | const {} DOTDOT {} const {}
        ;

unsigned_number
        : UNSIGNED_INTEGER {}
        | UNSIGNED_REAL {}
        ;

/* *********************************************** */

sign
        : PLUS {}
        | MINUS {}
        ;

typed_const
        : const {}
        | array_const {}
        | record_const {}
 /*       | set_const {}  */
        ;

array_const
        : LPAREN  typed_const_list {} RPAREN {}
        ;

typed_const_list
        : typed_const {} COMMA {} typed_const {}  /*was nuetzt ein Array mit nur einem Element? */
        | typed_const_list {} COMMA {} typed_const {}
        ;

record_const
        : LPAREN  const_field_list {} RPAREN {}
        ;

const_field_list
        : const_field {}
        | const_field_list {} SEMICOLON {} /*Fehler in tp4.y*/ const_field {}
        ;

const_field
        : const_field_name {} COLON {} typed_const {}
        ;

const_field_name
        : IDENTIFIER {}
        ;

set_const
        : LBRAC {} const_elem_list {} RBRAC {}
        ;

/***********************************************************************/
/* Type declarations: */

type_decl
        : IDENTIFIER {} EQUAL {} type_decl_type {} SEMICOLON {}
        ;

/* type_decl_name : IDENTIFIER {} ; */

type_decl_type
        : type {}
        | object_type {}
        ;
      
type    : simple_type {}
        | pointer_type {}
        | structured_type {}
        | string_type {}
        | procedural_type {}
        ;

simple_type
        : simple_type_IDENTIFIER {}
        | const_simple_expr {} DOTDOT {} const_simple_expr {}      	/* subrange */
        | LPAREN  enumeration_id_list {} RPAREN {}	/* enumeration */
        ;

simple_type_IDENTIFIER : IDENTIFIER {} ;

enumeration_id_list
        : enumeration_id {} COMMA {} enumeration_id {}
        | enumeration_id_list {} COMMA {} enumeration_id {}
        ;

enumeration_id : IDENTIFIER {} ;

pointer_type
        : UPARROW {} IDENTIFIER {} ;
        ;

structured_type
        : unpacked_structured_type {}
        | _PACKED_ {} unpacked_structured_type {}
        ;

unpacked_structured_type
        : array_type {}
        | record_type {}
        | set_type {}
        | file_type {}
        ;

array_type
        : _ARRAY_ {} LBRAC {} simple_type_list {} RBRAC {} _OF_ {} type {}
        ;

simple_type_list
        : simple_type {}
        | simple_type_list {} COMMA {} simple_type {}
        ;

record_type
        : _RECORD_ {} field_list {} _END_ {}
        ;

field_list    /* am Ende koennte auch ein Semicolon stehen! */
        : fixed_part {}
        | variant_part {}
        | fixed_part_2 SEMICOLON variant_part {}
        ;

fixed_part
        : fixed_part_2 {}
        | fixed_part_2  SEMICOLON {}
        ;

fixed_part_2
        : record_section {}
        | fixed_part_2 SEMICOLON record_section {}
        ;

record_section
        : /* empty  {}
        | */ record_section_id_list {} COLON {} type {}
        ;

record_section_id_list
        : record_section_id {}
        | record_section_id_list {} COMMA {} record_section_id {}
        ;

record_section_id : IDENTIFIER {} ;

variant_part
        : _CASE_ {} tag_field {} _OF_ {} variant_list {}
        ;

tag_field
        : tag_field_name {}
        | tag_field_name {} COLON {} tag_field_typename {}
        ;

tag_field_name : IDENTIFIER {} ;

tag_field_typename : IDENTIFIER {} ;

variant_list
        : variant_list_2 {}
        | variant_list_2  SEMICOLON {}
        ;

variant_list_2
        : variant {}
        | variant_list_2  SEMICOLON  variant {}
        ;

variant : case_tag_list {} COLON {} LPAREN {} variant_field_list {} RPAREN {}
        ;

variant_field_list
        : /* empty */ {}
        | field_list {}
        ;

case_tag_list
        : const {}
        | case_tag_list {} COMMA {} const {}
        ;

set_type
        : _SET_ {} _OF_ {} simple_type {}
        ;

file_type
        : _FILE_ {} _OF_ {} type {}
        | _FILE_ {}
        ;

string_type
        : _STRING_ {}
        | _STRING_ {} LBRAC {} const {} RBRAC {}
        ;

procedural_type
        : procedural_type_decl {}
        | procedural_type_decl {} _OF_ _OBJECT_ {}
        ;

procedural_type_decl
        : _PROCEDURE_ {} fp_list {}
        | _FUNCTION_ {} fp_list {} COLON {} fptype {}
        ;

object_type
        : new_object_type {}
        | old_object_type {}
        ;

/*ObjektTyp:             tp 1.0 f.W.
-- OBJECT ----------- komponentenliste -------------------------------- END --->
           |- Erbe -|                  |- PRIVATE - komponentenliste -|                */
old_object_type
        : _OBJECT_ {} oot_successor {} oot_component_list {} oot_privat_list {} _END_ {}
        ;

oot_privat_list
        :  /* empty */ {}
        | _PRIVATE_ {} oot_component_list {}
        ;

/*Komponentenliste:
--------------------------------------->
 |- Feldliste -|  |- Methodenliste -|            */
oot_component_list
        : /* empty */ {}
        | oot_field_list {}
        | oot_field_list {} oot_method_list {}
        | oot_method_list {}
        ;

/*Erbe:
-- ( -- Objekttypenbezeichner -- ) --->  */
oot_successor
        : LPAREN {} oot_typeidentifier {} RPAREN {}
        ;

oot_typeidentifier
        : IDENTIFIER {}
        ;

/*Feldliste:
--- Bezeichnerliste -- : -- Typ -- ; --->
 |<-----------------------------------<|  */
oot_field_list
        : oot_field {}
        | oot_field_list {} oot_field {}
        ;

oot_field
        : oot_id_list {} COLON {} type {} SEMICOLON {}
        ;

oot_id_list
        : oot_field_identifier {}
        | oot_id_list {} COMMA {} oot_field_identifier {}
        ;

oot_field_identifier
        : IDENTIFIER {}
        ;

/*Methodenliste:
--- Methodenkopf ------------------- ; --->
 |                |- ; - virtual -|     |
 |                                      |
 |<------------------------------------<|    */
oot_method_list
        : oot_method {}
        | oot_method_list {} oot_method {}
        ;

oot_method 
        : oot_method_head {}
        | oot_method_head {} _VIRTUAL_ {} SEMICOLON {}
        ;

/* Methodenkopf:
---- Prozedurkopf ------->
 |-- Funktionskopf ---|
 |-- Construktorkopf -|
 |-- Destruktorkopf --|           */ 
oot_method_head
        : proc_heading {}
        | func_heading {}
        | oot_constructor_head {}
        | oot_destructor_head {}
        ;

oot_constructor_head
        : _CONSTRUCTOR_ {} proc_name {} fp_list {} SEMICOLON {}
        ;

oot_destructor_head
        : _DESTRUCTOR_ {} proc_name {} fp_list {} SEMICOLON {}
        ;

/* Delphi 1.0 */
new_object_type
        : not_class_reference_type {}
        | not_object_type
        ;

/*class reference type:
-- CLASS -- OF -- object type identifier ---->       */
not_class_reference_type
        : _CLASS_ _OF_ {} not_object_type_identifier {}
        ;

not_object_type_identifier
        : IDENTIFIER {}
        ;

/*objecttype:
-- CLASS ------------------------------------------------------>
          |- heritage -| |--- component list ---------- END -|
                           |<- visibility specifier <-|             */
not_object_type
        : _CLASS_ {}					/* Vorwaertsdefinition */
        | _CLASS_ not_heritage {} 
        | _CLASS_ not_component_list_seq {} _END_ {}	/* direkte Ableitung von TObject */
        | _CLASS_ not_heritage {} not_component_list_seq {} _END_ {}
        ;
/* ACHTUNG::   tf1 = class ; fuehrt zu einem Compiler-Fehler 125:
 Undefinierte Klasse in vorhergehender Deklaration (Undefined Class in Preceding Declaration)
(Vorwaertsbezug in selber Typdeklaration)*/
/* class(...) end; ist aber moeglich!*/

not_component_list_seq
        : not_component_list {}
        | not_component_list_seq {} not_visibility_specifier {} not_component_list {}
        ;

/*heritage:
--- ( -- object type identifier -- ) ---->           */
not_heritage
        : LPAREN {} not_object_type_identifier {} RPAREN {}
        ;

/*visibility specifier:
--- PUBLISHED ------->
 |- PUBLIC ----|
 |- PROTECTED -|
 |- PRIVATE ---|                  */
not_visibility_specifier
        : _PUBLISHED_ {}
        | _PUBLIC_ {}
        | _PROTECTED_ {}
        | _PRIVATE_ {}
        ;

/*component list:
---------------------------------------------------------->
 |<- field definition <-| | |- method definition ---|  |
                          | |- property definition -|  |
                          |<--------------------------<|                     */
not_component_list
        : /* empty */ {}
        | not_component_list_1 {}
        | not_component_list_2 {}
        | not_component_list_1 {} not_component_list_2 {}
        ;
not_component_list_1
        : not_field_definition {}
        | not_component_list_1 {} not_field_definition {}
        ;

not_component_list_2
        : not_method_definition {}
        | not_property_definition {} 
        | not_component_list_2 {} not_method_definition {}
        | not_component_list_2 {} not_property_definition {}
        ;

/*field definition:
-- identifier list -- : -- type -- ; ---->             */
not_field_definition
        : not_field_identifier_list {} COLON {} type {} SEMICOLON {}
        ;

not_field_identifier_list
        : not_field_identifier {}
        | not_field_identifier_list {} COMMA {} not_field_identifier {}
        ;

not_field_identifier
        : IDENTIFIER {}
        ;

/*method definition:
-- method heading -- ; -- method directives ---->            */
not_method_definition
        : not_method_heading {} /*SEMICOLON {}*/ not_method_directives {}
        ;

/*method heading:
------------------ procedure heading ------->
 | |- CLASS -|  |- function heading ----|
 |                                      |
 |---------------- constructor heading -|
                |- destructor heading --|                   */
not_method_heading                 /*SEMICOLON gehoert dazu !*/
        : _CLASS_ {} proc_heading {}
        | _CLASS_ {} func_heading {}
        | func_heading {}
        | proc_heading {}
        | not_constructor_heading {}
        | not_destructor_heading {}
        ;

/*constructor heading: destructor heading:
-- CONSTRUCTOR --- identifier ----------------------------------------------------->
                |- qualified method identifier -| |- formal parameter list -|                */
not_constructor_heading
        : _CONSTRUCTOR_ {} qualified_identifier {} fp_list {} SEMICOLON {}
        ;

qualified_identifier
        : IDENTIFIER {} DOT {} IDENTIFIER {}
        | IDENTIFIER {}
        ;

not_destructor_heading
        : _DESTRUCTOR_ {} qualified_identifier {} fp_list {} SEMICOLON {} 
        ;

/*method directives:
-----------------------------------------------------------------------------  ------------------>
 |- VIRTUAL ----------------------- ; -| |- CDECL -- ; -| |- EXPORT -- ; -| |- ABSTRACT -- ; -|
 |- DYNAMIC ---------------------|                                                            |
 |- MESSAGE -- integer constant -|                                                            |
 |-- OVERRIDE ---------------------------------------------------------------------- ; -------|
override ersetzt virtual u. dynamic   */
not_method_directives
        : /* empty */
        | _VIRTUAL_ {} SEMICOLON {} not_method_directives_2 {}
        | _DYNAMIC_ {} SEMICOLON {} not_method_directives_2 {}
        | _MESSAGE_ {} integer_const {} SEMICOLON {} not_method_directives_2 {}
        | _OVERRIDE_ {} SEMICOLON {}
        ; 

not_method_directives_2
        : not_method_directives_cdecl {} not_method_directives_export {} not_method_directives_abstract {}
        ;

not_method_directives_cdecl
        : /* empty */ {}
        | _CDECL_ {} SEMICOLON {}
        ;

not_method_directives_export
        : /* empty */ {}
        | _EXPORT_ {} SEMICOLON {}
        ;

not_method_directives_abstract
        : /* empty */ {}
        | _ABSTRACT_ {} SEMICOLON {}
        ;

integer_const
        : sign UNSIGNED_INTEGER {}
        | UNSIGNED_INTEGER {}
        | IDENTIFIER {}
        | sign {} IDENTIFIER {}
        ;

/******************************************************************************/
/*property definition:
-- PROPERTY -- identifier ------------------------ property specifiers -- ; -->
                          |- property interface -|                               */
not_property_definition
        : _PROPERTY_ {} IDENTIFIER {} not_property_interface {} not_property_specifiers {}
            SEMICOLON {} not_array_defaultproperty {}
        ;


not_array_defaultproperty
        : /* empty */
        | _DEFAULT_ {} SEMICOLON {} /* Array-Standardeigenschaft */
        ;

/*property interface:
------------------------------ : -- type identifier ----------------------------------->
 |- property parameter list -|                       |- INDEX -- integer constant -|          */
not_property_interface
        : /* empty */ {}
        | not_property_parameter_list {} COLON {} not_type_identifier {} not_property_interface_index {}
        ;

not_type_identifier
        : IDENTIFIER {}
        | _STRING_ {}
        ;

not_property_interface_index
        : /* empty */ {}
        | _INDEX_ {} integer_const {}
        ;

/*property parameter list:
-- [ --- parameter declaration --- ] ----->
      |<-------- ; -------------|                                     */
not_property_parameter_list
        : /* empty */ {}
        | LBRAC {} not_parameter_decl_list {} RBRAC {}
        ;

not_parameter_decl_list
        : not_parameter_decl {}
        | not_parameter_decl_list {} SEMICOLON {} not_parameter_decl {}
        ;

not_parameter_decl
        : not_parameter_name_list {} COLON {} fptype {}
        ;

not_parameter_name_list
        : not_parameter_name {}
        | not_parameter_name_list {} COMMA {} not_parameter_name {}
        ;

not_parameter_name
        : IDENTIFIER {}
        ;

/*property specifiers:
--------------------------------------------------------------------------------------------->
 |- read specifier -| |- write specifier -| |- stored specifier -| |- default specifier -|           */
not_property_specifiers
        : not_read_specifier {} not_write_specifier {} not_stored_specifier {} not_default_specifier {}
        ;

/*default specifier:
--- DEFAULT -- constant ----->
 |- NODEFAULT ------------|              */
not_default_specifier
        : /* empty */ {}
        | _DEFAULT_ {} const {}
        | _NODEFAULT_ {} 
        ;

/*stored specifier:
-- STORED --- field or method ------------>
           |- boolean constant --|                    */
not_stored_specifier
        : /* empty */ {}
/*        | _STORED_ not_field_or_method {} */
           /* muss in not_stored_bool_const extra behandelt werden */
        | _STORED_ {} not_stored_bool_const {}
        ;

not_stored_bool_const 
        : const {}               /* berechnete const */
        ;

/*read specifier:
-- READ -- field or method ---->             */
not_read_specifier
        : /* empty */ {}
        | _READ_ {} not_field_or_method {}
        ;

/*write specifier:
-- WRITE -- field or method ---->               */
not_write_specifier
        : /* empty */ {}
        | _WRITE_ {} not_field_or_method {}
        ;

/*field or method:
--- field identifier ------->
 |- method identifier --|                */
not_field_or_method
        : IDENTIFIER {}
        ;

/***********************************************************************/
/* Variable declarations: */

var_decl
        : var_name_list {} COLON {} type {} absolute_clause {} SEMICOLON {}
        ;

var_name_list
        : var_name {}
        | var_name_list {} COMMA {} var_name {}
        ;

var_name : IDENTIFIER {} ;

absolute_clause
        : /* empty */ {}
        | _ABSOLUTE_ {} UNSIGNED_INTEGER {} COLON {} UNSIGNED_INTEGER {}
        | _ABSOLUTE_ {} declared_var_name {}
        ;

declared_var_name : IDENTIFIER {} ;

/***********************************************************************/
/* Constructor and destructor declarations: */

constructor_decl
        : not_constructor_heading {} not_constructor_block_decl /* oder proc_block */ {}
        ;

destructor_decl 
        : not_destructor_heading {} not_constructor_block_decl /* oder proc_block; inline ?? */ {}
        ;

not_constructor_block_decl
        : block {}
        | external_directr {}
        | asm_block {}
        ;

/***********************************************************************/
/* Procedure and function declarations: */

proc_decl
        : proc_heading {} proc_block {}
        ;

func_decl
        : func_heading {} func_block {}
        ;

proc_heading
        : _PROCEDURE_ {} proc_name {} fp_list {} SEMICOLON  {}
        ;

proc_name
        : proc_name {} DOT {} IDENTIFIER {} /* Object Method, qualified name */
        | IDENTIFIER {}
        ;

func_heading
        : _FUNCTION_ {} func_name {} fp_list {} COLON {} fptype {} SEMICOLON {}
        ;

func_name
        : func_name {} DOT {} IDENTIFIER {} /* Object Method, qualified name */
        | IDENTIFIER {}
        ;

/* --------------------------------------- block ------>
    |- near --------- ; --|    |- external directr -|
    |- far --------|           |- asm Block --------|
    |- export -----|           |- forward ----------|
    |- cdecl ------|                                |
    |- interrupt --|                                |
    |                                               |
    |---------------- inline directr ---------------|           */
proc_block
        : proc_block_prefix {} proc_block_decl {}
        | inline_directr {} SEMICOLON {}
        ;

proc_block_prefix
        : func_block_prefix 
        | _INTERRUPT_ {} SEMICOLON {} /* aber nur fuer proc mit best. Parametern */
        ;

func_block
        : func_block_prefix {} proc_block_decl {}
        | inline_directr {} SEMICOLON {}
        ;

func_block_prefix
        : /* empty */ {}
        | _NEAR_ {}  SEMICOLON {}
        | _FAR_ {}  SEMICOLON {}
        | _EXPORT_ {}  SEMICOLON {}
        | _CDECL_ {}  SEMICOLON {}
        ;

proc_block_decl
        : block {}
        | external_directr {}
        | asm_block {}
        | _FORWARD_ {} SEMICOLON {}
        ;

/* external directr :
  - EXTERNAL ------------------------------------------->
               |- StringConst -----------------------|
                               |- NAME StringConst --|
                               |- INDEX integerConst-|            */
external_directr
        : _EXTERNAL_ {} SEMICOLON {}
        | _EXTERNAL_ {} string_const_2 {} external_directr_2 {} 
        ;

external_directr_2
        : SEMICOLON {}
        | _NAME_  {} string_const_2 {} SEMICOLON {}
        | _INDEX_ {} external_directr_3 {} SEMICOLON {}
        ; 

external_directr_3
        : UNSIGNED_INTEGER {} 
        | MINUS {} UNSIGNED_INTEGER {}
        ;

string_const_2
        : IDENTIFIER {}
        | STRING_CONST {}
        ;

asm_block : _ASSEMBLER_ {} SEMICOLON {} impl_decl_sect_list {} asm_stmt {} SEMICOLON {};

/* inline_stmt: -- INLINE -- ( --- inline element --- ) -->
                                |<---- / -------<--|         */
inline_directr : _INLINE_ {} LPAREN {} inline_element {} RPAREN {} ;

/* inline_element: --------- constant ---------------------------------->
                    |- < -|                                          |
                    |- > -|                                          |
                    |-- variable identifier -------------------------|
                                             |-- sign -- constant --|
                                              |---------<----------|     */
inline_element
        : inline_param {}
        | inline_element {} SLASH {} inline_param {} 
        ;

inline_param
        : LT {} inline_const {}
        | GT {} inline_const {}
        | inline_param_variable {}
        ;

inline_param_variable
        : variable_reference {} 
        | inline_param_variable {} sign {} inline_const {}
        ;

inline_const
        : const_factor {}
        ;

block   : impl_decl_sect_list {} compound_stmt {} SEMICOLON {}
        ;

fp_list : /* empty */ {}
        | LPAREN {} fp_sect_list {} RPAREN {}
        ;

fp_sect_list
        : fp_sect {}
        | fp_sect_list {} SEMICOLON {} fp_sect {}
        ;

fp_sect : param_name_list {} COLON {} fptype_new {}            /* Werteparameter */
        | _VAR_ param_name_list {} COLON {} fptype_new {}      /* Variablenparameter getypt */
            /* auch 'var a: array of const' ist hier gueltig ! */
        | _VAR_ param_name_list {}                             /* untyped VAR-Parameter */
        | _CONST_ {} param_name_list {} COLON {} fptype_new {} /* Constant Parameter */
        ;

param_name_list
        : param_name {}
        | param_name_list {} COMMA {} param_name {}
        ;

param_name
        : IDENTIFIER {}
        ;

fptype                          /* old style */
        : func_res_type_name {}
        | _STRING_ {}
        ;
fptype_new
        : fptype {}
        | _ARRAY_ _OF_ {} fptype {} /* open-array Parameter */
        | _ARRAY_ _OF_ _CONST_ {}
        ;

func_res_type_name : IDENTIFIER {}

/***********************************************************************/
/* Statements: */

stmt    : unlabelled_stmt {}
        | label {} COLON {} unlabelled_stmt {}
        ;

unlabelled_stmt
        : /* empty */ {}
        | assignment {}        	/* Zuweisung */
        | proc_call {}		/* Prozedur- oder Funktionsaufruf */
        | goto_stmt {}		/* GOTO-Anweisung */
        | compound_stmt {}	/* Verbundanweisung */
        | if_stmt {}		/* IF-Anweisung */
        | case_stmt {}		/* CASE-Anweisung */
        | repeat_stmt {}
        | while_stmt {}
        | for_stmt {}
        | with_stmt {}
        | asm_stmt {}		/* Assembler-Block */
        | inline_directr {}	/* Inline-Anweisung */
        | inherited_message {} 	/* Aufruf der vererbten Botschaftsbehandlungsroutine */
        | try_stmt {}         /* Ressourcenschutzanweisung */
        | raise_stmt {}       /* Exception ausloesen */
        ;

assignment
        : variable_reference {} _ASSIGNMENT_ {} expr {}
          /* type cast */
        ;

proc_call
        : variable_reference {}
          /* type cast */
          /*oder Procedure-Call; funktioniert jetzt mit inc(k)*/
          /*also procedural type TypeCast; or function delivering a procedural type */
        ;

goto_stmt
        : _GOTO_ {} label {}
        ;

compound_stmt
        : _BEGIN_ {} stmt_list {} _END_ {}
        ;

stmt_list
        : stmt {}
        | stmt_list SEMICOLON stmt {}
        ;

if_stmt : _IF_ {} expr {} if_then_else_branch
        ;

if_then_else_branch
        : _THEN_ then_branch {}
        | _THEN_ then_branch _ELSE_ {} else_branch {}
        ;

then_branch
        : stmt {}
        ;

else_branch
        : stmt {}
        ;

case_stmt
        : _CASE_ {} expr {} _OF_ {} case_list {} else_case {} _END_ {}
        ;

case_list
        : case {}
        | case_list {} SEMICOLON {} case {}
        ;

case    : /* empty */ {}
        | case_label_list {} COLON {} stmt {}
        ;

case_label_list
        : case_label {}
        | case_label_list {} COMMA {} case_label {}
        ;

case_label
        : const {}
        | const {} DOTDOT {} const {}
        ;

else_case
        : /* empty */ {}
        | _ELSE_ {} stmt_list {}
        ;

repeat_stmt
        : _REPEAT_ {} stmt_list {} _UNTIL_ {} expr {}
        ;

while_stmt
        : _WHILE_ {} expr {} _DO_ {} stmt {}
        ;

for_stmt
        : _FOR_ {} variable_reference {}
               /* eigentlich hier 'variable', da auch Arrays akzeptiert werden */
           _ASSIGNMENT_ {} expr {}
           for_stmt_2 {}
        ;

for_stmt_2
        : _TO_ {} expr {} _DO_ {} stmt {}
        | _DOWNTO_ {} expr {} _DO_ {} stmt {}
        ;       

with_stmt
        : _WITH_ {} variable_list {} _DO_ {} stmt {}
        ;

variable_list
        : variable_reference {}  /* acessing func.recordfield not allowed here */
        | variable_list {} COMMA {} variable_reference {}
        ;

inherited_message
        : _INHERITED_ {} /* Aufruf der vererbten Botschaftsbehandlungsroutine */
        ;

try_stmt
        : _TRY_ {} stmt_list {} try_stmt_2 {} 
        ;

try_stmt_2
        : _FINALLY_ {} stmt_list {} _END_ {} 
        | _EXCEPT_ {} exception_block {} _END_ {}
        ;

exception_block
        : exception_handler_list {} exception_block_else_branch {}
 /*optionales SEMICOLON an dieser ^^^ Stelle moeglich*/
        | exception_handler_list SEMICOLON {} exception_block_else_branch {}
        | stmt_list {}
        ;

exception_handler_list
        : exception_handler {}
        | exception_handler_list SEMICOLON {} exception_handler {}
        ;

exception_block_else_branch
        : /* empty */ {}
        | _ELSE_ {} stmt_list {}
        ;

exception_handler
        : _ON_ {} exception_identifier {} _DO_ {} stmt {}
        ;

exception_identifier
        : exception_class_type_identifier {}
        | exception_variable {} COLON {} exception_class_type_identifier {}
        ;

exception_class_type_identifier
        : IDENTIFIER {}
        ;

exception_variable
        : IDENTIFIER {}
        ;

raise_stmt
        : _RAISE_ {} /* nur innnerhalb des exception_blocks erlaubt ! */
            /* sonst: Compiler-Fehler 198: RAISE nicht auerhalb von EXCEPT..END erlaubt */
        | _RAISE_  variable_reference {}
        | _RAISE_  variable_reference {} _AT_ {} raise_at_stmt_2 {}
        ;

raise_at_stmt_2             /*konstanter Adressausdruck; z.B. Bezeichner */ 
        : variable_reference {}
        | _NIL_ {}
        ;

/***********************************************************************/
/* Variables:                 |<-----------------------<|  
----------------- IDENTIFIER -------------------------------------------------------->
 |- INHERITED -|               |- [ -- expr list -- ] -|  
                               |-- . -- IDENTIFIER ----|   
                               |-------- ^ ------------|                               
                               |- ( -- expr list -- ) -|   {als Function-Call}        */

variable_reference
        : APE_SYMBOL {} 
        | variable_reference {} APE_SYMBOL {}
        | variable_reference {} variable {}
        | variable {}
        ;

variable               /* Achtung: kann auch von proc_call kommen !! */
        : _INHERITED_ {} variable_IDENTIFIER {}
        | variable_IDENTIFIER {}     /*Var_name; auch type oder func_call moeglich hier; qualified name */
        | variable  variable_2 {}
        ;

variable_IDENTIFIER 
        : IDENTIFIER {}
        ;

variable_2
        : LBRAC {} expr_list {} RBRAC {}  /* array component */
        | DOT {} IDENTIFIER {}         /* record field; also: QualifiedName, func_call */
        | UPARROW {}  			       /* pointer value */
        | LPAREN {} expr_list {} RPAREN {}
           /* TypeCast */
           /*oder Function-Call */
           /*oder von proc_call in stmt  */
           /*also procedural type TypeCast; or function delivering a procedural type */
        ;

expr_list
        : expr {}
        | expr_list {} COMMA {} expr {}
        ;


/***********************************************************************/
/* Expressions: */

expr    : simple_expr {}
        | simple_expr {} relop {} simple_expr {}
        | simple_expr  COLON  simple_expr {} 
 /* Formatiert eine Zahl in einen String; benutzt nur in Write, Writeln, Str  */
        | simple_expr  COLON  simple_expr {} COLON {} simple_expr {}
        ;


relop   : EQUAL {} | NOTEQUAL {} | LT {} | GT {} | LE {} | GE {} | _IN_ {} | _IS_ {}
        ;

simple_expr
        : term {}
        | simple_expr {} addop {} term {}
        ;

addop   : PLUS {} | MINUS {} | _OR_ {} | _XOR_ {}
        ;

term    : factor {}
        | term {} mulop {} factor {}
        ;

mulop   : STAR {} | SLASH {} | _DIV_ {} | _MOD_ {} | _SHL_ {} | _SHR_ {} | _AND_ {} | _AS_ {}
        ;

/* Parameterless function calls, and function calls looking like type
   casts are caught as variables. */

factor  : variable_reference {}
        | unsigned_number {}
        | string {}
        | LBRAC {} elem_list {} RBRAC {}
           /* could be also an actual parameter for 'array of const'; 'expr DOTDOT expr' 
              would then be not allowed here */
        | _NIL_ {}
        | parenth_factor {}
        | _NOT_ {} factor {}
        | sign {} factor {}
        ;

parenth_factor
        : LPAREN {} expr {} RPAREN {}
        | parenth_factor {} UPARROW {}
        ;

elem_list
        : /* empty */ {}
        | elem_list1 {}
        ;

elem_list1
        : elem {}
        | elem_list1 {} COMMA {} elem {}
        ;

elem    : expr {}
        | expr {} DOTDOT {} expr {}
        ;

/***********************************************************************/
/*Old Inheritance of TurboPascal*/
/* Such a construct is used in TVision\editors.pas as:
   const
     FirstKeys: array[0..37 * 2] of Word = (37,
       Ord(^A), cmWordLeft, Ord(^C), cmPageDown,
       Ord(^D), cmCharRight, Ord(^E), cmLineUp,
                                                          ...
 DON'T ask me, what this means !!!
 (Delphi 1.0 accepts Ord(^a) as well; seems that Identifier must be one char that corresponds
 to a character-table??;
 the Identifier can be also almost any sign: '^<' would be valid too !!;
 type of ^Identifier is char !! )                                                       */

string  : STRING_CONST {}
        | UPARROW {SwitchState (Normal,CharTable);} IMMCHAR_ID  /*is any char*/
          {}
        ;

/* *************************************** */
/* Link zum Assembler-Parser */

asm_stmt
        : _ASM_ { (* AsmParse.Run_Parse; *) } _END_ {}
        ;

%%

type TBuf = array[1..10240] of char;
var Buf : ^TBuf;


procedure Run_Parse;
begin
  filename := paramStr(1);
  if filename='' then
    begin
      write('input file: ');
      readln(filename);
    end;
  assign(yyinput, filename);
  reset(yyinput);

  New( Buf );
  SetTextBuf( yyinput, Buf );

  if yyparse=0 then writeln('successful parse!');

  Flush(yyoutput);

end;

begin
end.
