
(* lexical analyzer template (TP Lex V3.0), V1.0 3-2-91 AG *)

(* global definitions: *)
(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*                                                                  *)
(* (c) rr, 9.9., 19.9.,                                             *)
(* ---------------------------------------------------------------- *)

unit Scanner;

interface

function yylex : Integer;

{$R Scanner.Res}


const Normal = 2;
const String1 = 4;
const String2 = 6;

const asmString1 = 8;
const asmString2 = 10;
const asmState = 12;

const OverreadState = 14;

const CompilerSwitch = 16;

const Comment = 18;

const CharTable = 20;


  implementation
  uses SysUtils, Parser, Errors, LexLib, YaccLib, ScanStack;

  {$I Scanner.inc}

  CONST
    HexBase = 16;
    DecBase = 10;
    BinBase = 2;
    OctBase = 8;

  PROCEDURE RaiseScannerError( s : word; const Args : array of const );
    BEGIN
      writeln( LoadStr( s ) );
      { besser :
      raise ECompilerError.CreateError( @TTextRec(yyinput).Name, yylineno, yycolno, IDEM_Lex, s, Args );
      }
    END;

  PROCEDURE MakeInt (S: STRING; FixBase: BYTE);
    CONST
      Values: ARRAY [0..15] OF CHAR =
                ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
                 'A', 'B', 'C', 'D', 'E', 'F');
    VAR
      i:     INTEGER;
      v:     LONGINT;
      Error: BOOLEAN;
      Base:  LONGINT;


    FUNCTION Index (c: CHAR): BYTE;
      VAR j, i: BYTE;
    BEGIN
      j := 16;
      i := 0;
      FOR i := 0 TO 15 DO
        IF UpCase (c) = Values [i] THEN
          j := i;
      IF j > 15 THEN
        Error := TRUE;     { Zeichen ungltig ! }
      Index := j;
    END;

  BEGIN
    Error := FALSE;
    Base := FixBase;
    v := Index (s [Length (s)]) * 1;
    FOR i := Length (S)-1 DOWNTO 1 DO BEGIN
      v := v + Index (s [i]) * Base;
      Base := Base * FixBase;
    END;
    IF NOT Error THEN BEGIN
      yylVal.yyLongint := v;
      Return (UNSIGNED_INTEGER);
    END ELSE BEGIN
      RaiseScannerError (IDS_NumberTooBig, ['.']);
      Return (ILLEGAL);
    END;
  END;

  PROCEDURE Commenteof;
  BEGIN
    RaiseScannerError (IDS_UnexpectedEOFInsideComment, [yyLineNo,'.']
                       {'Unexpected EOF inside Comment at line ', yyLineNo} );
  END;

  FUNCTION Upper (Str: STRING): STRING;
    VAR i: INTEGER;
  BEGIN
    FOR i := 1 TO Length (str) DO
      str [i] := UpCase (str [i]);
    Upper := Str;
  END;

  FUNCTION isKeyword (ID: STRING; VAR Token: INTEGER): BOOLEAN;
    (* checks whether id is Pascal keyword; if so, returns corresponding
       token number in token *)
      (* table of Pascal keywords: *)

  {$I Keywords.inc}

    begin
      id := UpperCase(id);
      isKeyword := kwlookup( id, token );
    end(*isKeyword*);

  function isStdInstruction(id : string; var token : integer) : boolean;
    (* checks whether id is Pascal keyword; if so, returns corresponding
       token number in token *)
     (* table of Pascal Standard Instructions: *)

  {$I Std_Inst.inc}

  var res : boolean;
    begin
      id := UpperCase(id); {$B-}
      isStdInstruction := kwlookup( id, token ){ and Parser.WouldAcceptChar( token )};
    end(*isStdInstruction*);

  var CompSwitch : string;

  Const NEWLINE : string[2] = #13#10;
  Procedure Input2Output( s : string );
    BEGIN
      {$ifdef yydebug}
      write( s );
      {$endif}
    END;

      (* {digit}+(((\.{digit}+)([Ee][+-]?{digit}+)?)|([Ee][+-]?{digit}+))
       {D}+(\.{D}* )?([Ee][+-]?{D}+)?
      {digit}+(\.{digit}* )?([Ee][+-]?{digit}+)? *)


  VAR
    Result:  INTEGER;
    c:       CHAR;
    KW:      INTEGER;
    s:       STRING;
    i:       INTEGER;
    cs:      STRING [2];

 Procedure EvalCommentStart( oldState: integer);
 var c : char;
  BEGIN
    cs := yyText;
    Input2Output (yyText);
    c := get_char;
    if c = '$' then begin
      SwitchState(oldState,CompilerSwitch);
      CompSwitch := '';
    end else begin
      SwitchState(oldState,Comment);
      Unget_Char(c);
    end;
  END;

procedure yyaction ( yyruleno : Integer );
  (* local definitions: *)
begin
  (* actions: *)
  case yyruleno of
  1{<Normal>'}:
         
          BEGIN
            s := '';
            SwitchState (Normal,String2);{Start (String2);}
          END;
  2{<Normal>"#"}:
           
          BEGIN
            s := '';
            SwitchState (Normal,String1);{Start (String1);}
            Unget_Char (yyText [1]);
          END;

  3{<String1>'}:
          
          BEGIN
          Start (String2);
          END;
  4(*<String1>#{integer}*):
                   
          BEGIN   { #123 oder #$12 im String enthalten }
            Delete (yyText, 1, 1);
            Val (yyText, i, Result);
            IF Result = 0 THEN
              s := s + Char (i)
            ELSE BEGIN
              Return (UNSIGNED_INTEGER);
              RaiseScannerError (IDS_ErrorInIntegerConst, ['.']);
            END;
          END;
  5{<String1>.}:
          
          BEGIN
            Return2OldState;{Start (Normal);}
            Return (STRING_CONST);
            UnGet_Char (yytext [1]);
            Input2Output (''''+s+'''');
            {RaiseScannerError (IDS_ErrorInIntegerConst, ['.']);}
          END;

  6{<String2>"''"}:
             
          s := s+'''';
  7{<String2>'}:
          
          Start (String1);

  8{<asmState>'}:
           
          BEGIN
            SwitchState (asmState,asmString1);{Start (asmString1);} s := '';
          END;
  9{<asmString1>"''"}:
                
          s := s+'''';
  10{<asmString1>'}:
             
          BEGIN
            Return2OldState;{Start (asmState);}
            Return (STRING_CONST);
            Input2Output (''''+s+'''');
          END;

  11{<asmState>\"}:
            
          BEGIN
            SwitchState (asmState,asmString2);{Start (asmString2);} s := '';
          END;
  12{<asmString2>""""}:
                
          s := s+'"';
  13{<asmString2>\"}:
              
          BEGIN
            Return2OldState;{Start (asmState);}
            Return (STRING_CONST);
            Input2Output ('"'+s+'"');
          END;

  14{<String2,asmString1,asmString2>\n}:
                                 
          BEGIN
            RaiseScannerError (IDS_StringExceeedsLine, ['.']);  Halt;
          END;
  15{<String2,asmString1,asmString2>.}:
                                
          s := s + yyText;



  16{<Normal>"(*"|"{"}:
                
          BEGIN {State: Normal}
            EvalCommentStart(Normal);
          END;

  17{<asmState>"(*"|"{"}:
                  
          BEGIN
            EvalCommentStart(asmState);
          END;

  18(*<asmState>{identifier}*):
                      
          BEGIN
            Input2Output (yyText);
            {IF isAssemblerKeyword (yyText, KW) THEN
              Return (KW)
            ELSE IF isAssemblerRegister (yyText, KW) THEN
              Return (KW)
            ELSE }
              Return (IDENTIFIER);
            IF Upper (yyText) = 'END' THEN BEGIN
              Return2OldState; {Could only be Normal}
              Return (_END_);
            END;
          END;
  19(*<asmState>{asmLabel}*):
                    
          BEGIN
            Return (ASM_LABEL);
            Input2Output (yytext);
          END;
  20(*<asmState>{asmBinary}*):
                     
          BEGIN
            Input2Output (yyText);
            Dec (Byte (yyText [0]));
            MakeInt (yyText, BinBase);
          END;
  21(*<asmState>{asmOctal}*):
                    
          BEGIN
            Input2Output (yyText);
            Dec (Byte (yyText [0]));
            MakeInt (yyText, OctBase);
          END;
  22(*<asmState>{asmHex}*):
                  
          BEGIN
            Input2Output (yyText);
            Dec (Byte (yyText [0]));
            MakeInt (yyText, HexBase);
          END;
  23{<asmState>\n}:
            
          BEGIN
            Input2Output( NEWLINE );
            Return (ASM_SEPERATOR);
          END;
  24{<asmState>";"}:
             
          BEGIN
            Input2Output (';');
            Return (ASM_SEPERATOR);
          END;
  25(*<asmState>{asmWhite}*):
                    
          Input2Output (yyText);
  26(*<asmState>{integer}*):
                   
          BEGIN
            Val (yyText, yylVal.yyLongint, Result);
            Input2Output (yyText);
            IF Result=0 THEN
              Return (UNSIGNED_INTEGER)
            ELSE
              Return (ILLEGAL);
          END;
  27{<asmState>.}:
           
          BEGIN
            RaiseScannerError (IDS_IllegalChar, [yyText, '.']
                               {'Illegal character: ', yyText});
            Return (ILLEGAL);
          END;
  28(*<Normal>{realinteger}/[^\.]*):
                           
          BEGIN {State: Normal}             
            if (yyactchar = '.') then
              {hngt jetzt bei [1..2]; Verwechslung mit 1.2e4 abfangen}
              if (yylastchar = '.') then begin
                {hngt jetzt bei [1..2]; Verwechslung mit 1.2e4 abfangen}
                Val (copy(yyText,1,length(yytext)-1), yylVal.yyLongint, Result);
                Input2Output (copy(yyText,1,length(yytext)-1));
                IF Result=0 THEN BEGIN
                  Return (UNSIGNED_INTEGER);
                  unget_char( '.' );
                  exit;
                end else
                  Return (ILLEGAL);
              end else {z.B. '1.3e1..' oder '1.3..' }
                Return (ILLEGAL);
            {Normaler Real}
            Val (yyText, yylVal.yyExtended, Result);
            Input2Output (yyText);
            IF Result=0 THEN BEGIN
              Return (UNSIGNED_REAL);
            END ELSE
              Return (ILLEGAL);
          END;
  29(*<Normal>{integer}*):
                 
          BEGIN {State: Normal}
            Val (yyText, yylVal.yyLongint, Result);
            Input2Output (yyText);
            IF Result=0 THEN
              Return (UNSIGNED_INTEGER)
            ELSE
              Return (ILLEGAL);
          END;
  30(*<Normal>{identifier}*):
                    
          BEGIN {State: Normal}
            Input2Output (yyText);
            IF isKeyword(yyText, KW) THEN BEGIN
              Return (KW);
              IF KW = _ASM_ THEN
                SwitchState (Normal,asmState);
            END ELSE IF  isStdInstruction(yyText, KW) THEN
              if WouldAcceptChar(KW) then
                Return(KW)
              else
                Return (IDENTIFIER)
            ELSE
              Return (IDENTIFIER);
          END;
  31{<Normal>".."}:
            
          BEGIN
            Input2Output (yyText);
            Return (DOTDOT);
          END;
  32{<Normal>":="}:
            
          BEGIN
            Input2Output (yyText);
            Return (_ASSIGNMENT_);
          END;
  33{<Normal>"<="}:
            
          BEGIN
            Input2Output (yyText);
            Return (LE);
          END;
  34{<Normal>"<>"}:
            
          BEGIN
            Input2Output (yyText);
            Return (NOTEQUAL);
          END;
  35{<Normal>">="}:
            
          BEGIN
            Input2Output (yyText);
            Return (GE);
          END;
  36{<Normal>"(."}:
            
          BEGIN
            Input2Output (yyText);
            yyText := '[';
            Return (LBRAC);
          END;
  37{<Normal>".)"}:
            
          BEGIN {State: Normal}
            Input2Output (yyText);
            yyText := ']';
            Return (RBRAC);
          END; 
  38(*<Normal>{eof}*):
             
          BEGIN {State: Normal}
            Input2Output (yyText);
            Return (0);
          END;
  39(*<Normal>{white}*):
               
          BEGIN {State: Normal}
            IF yyText = #10 THEN
              Input2Output ( NEWLINE )
            ELSE
              Input2Output (yyText);
          END;

  40{<asmState,Normal>"+"}:
                    
          Return(PLUS);
  41{<asmState,Normal>"-"}:
                    
          Return(MINUS);
  42{<asmState,Normal>"*"}:
                    
          Return(STAR);
  43{<asmState,Normal>"/"}:
                    
          Return(SLASH);
  44{<asmState,Normal>"="}:
                    
          Return(EQUAL);
  45{<asmState,Normal>"<"}:
                    
          Return(LT);
  46{<asmState,Normal>">"}:
                    
          Return(GT);
  47{<asmState,Normal>","}:
                    
          Return(COMMA);
  48{<asmState,Normal>"("}:
                    
          Return(LPAREN);
  49{<asmState,Normal>")"}:
                    
          Return(RPAREN);
  50{<asmState,Normal>":"}:
                    
          Return(COLON);
  51{<asmState,Normal>";"}:
                    
          Return(SEMICOLON);
  52{<asmState,Normal>"@"}:
                    
          Return(APE_SYMBOL);
  53{<asmState,Normal>"^"}:
                    
          Return(UPARROW);
  54{<asmState,Normal>"["}:
                    
          Return(LBRAC);
  55{<asmState,Normal>"]"}:
                    
          Return(RBRAC);
  56{<asmState,Normal>"."}:
                    
          Return(DOT);
  57{<asmState>"&"}:
             
          Return(ET);

  58{<Normal>.}:
         
          BEGIN {State: Normal}
            RaiseScannerError (IDS_IllegalChar, [yyText, '.']);
            Return (ILLEGAL);
          END;

  59{<OverreadState>"(*"|"{"}:
                       
          BEGIN
            EvalCommentStart(OverreadState);
          END;
  60{<OverreadState>\0}:
                 
          BEGIN
            Return2OldState; {Could be Normal or AsmState}
            RaiseScannerError( IDS_CS_expected, ['.'] );
          END;
  61{<OverreadState>'}:
                
          BEGIN
            s := '';
            if FormerState = Normal then
              SwitchState (OverreadState, String2)
            else if FormerState = asmState then
              SwitchState (OverreadState,asmString1)
            else
              RaiseScannerError( IDS_InternalScannerError, ['.'] );
          END;
  62{<OverreadState>\"}:
                 
          BEGIN
            if FormerState = Normal then
            else if FormerState = asmState then begin
              SwitchState (OverreadState,asmString2); s := '';
            end else
              RaiseScannerError( IDS_InternalScannerError, ['.'] );
          END;
  63{<OverreadState>.}:
                
          BEGIN
            Input2Output (yyText);
          END;


  64{<Comment>"*)"|""}:
                 
          BEGIN
            IF ((yyText = '}') AND (cs = '{')) OR
               ((yyText = '*)') AND (cs = '(*')) THEN BEGIN
              Input2Output (yyText);
              Return2OldState; {Could be Normal or AsmState}
            END;
          END;
  65{<Comment>\0}:
           
          BEGIN
            Return2OldState; {Could be Normal or AsmState or OverreadState}
            CommentEof;
          END;
  66{<Comment>.}:
          
          BEGIN
            Input2Output (yyText);
          END;


  67{<CompilerSwitch>\0}:
                  
          BEGIN
            Return2OldState; {Could be Normal or AsmState or OverreadState}
            RaiseScannerError( IDS_CS_expected, ['.'] );
          END;
  68{<CompilerSwitch>"*)"|""}:
                        
          BEGIN
            IF ((yyText = '}') AND (cs = '{')) OR
               ((yyText = '*)') AND (cs = '(*')) THEN BEGIN
              Input2Output (yyText);
              {EvalCompilerSwitch( CompSwitch );}
              Return2OldState; {Could be Normal or AsmState or OverreadState}
            END;
          END;
  69{<CompilerSwitch>.}:
                 
          BEGIN
            Input2Output (yyText);
            CompSwitch := CompSwitch + yyText ;
          END;

  70{<CharTable>.}:
            
          BEGIN
            Input2Output (yyText);
            s := yyText;
            Return (IMMCHAR_ID);
            Return2OldState;           
          END;
  end;
end(*yyaction*);

(* DFA table: *)

type YYTRec = record
                cc : set of Char;
                s  : Integer;
              end;

const

yynmarks   = 174;
yynmatches = 174;
yyntrans   = 250;
yynstates  = 140;

yyk : array [1..yynmarks] of Integer = (
  { 0: }
  { 1: }
  { 2: }
  { 3: }
  { 4: }
  { 5: }
  { 6: }
  { 7: }
  { 8: }
  { 9: }
  { 10: }
  12,
  { 11: }
  12,
  { 12: }
  { 13: }
  { 14: }
  { 15: }
  { 16: }
  { 17: }
  { 18: }
  { 19: }
  { 20: }
  { 21: }
  { 22: }
  1,
  58,
  { 23: }
  2,
  58,
  { 24: }
  48,
  58,
  { 25: }
  16,
  58,
  { 26: }
  29,
  58,
  { 27: }
  58,
  { 28: }
  30,
  58,
  { 29: }
  56,
  58,
  { 30: }
  50,
  58,
  { 31: }
  45,
  58,
  { 32: }
  46,
  58,
  { 33: }
  38,
  { 34: }
  39,
  { 35: }
  40,
  58,
  { 36: }
  41,
  58,
  { 37: }
  42,
  58,
  { 38: }
  43,
  58,
  { 39: }
  44,
  58,
  { 40: }
  47,
  58,
  { 41: }
  49,
  58,
  { 42: }
  51,
  58,
  { 43: }
  52,
  58,
  { 44: }
  53,
  58,
  { 45: }
  54,
  58,
  { 46: }
  55,
  58,
  { 47: }
  39,
  58,
  { 48: }
  58,
  { 49: }
  3,
  5,
  { 50: }
  5,
  { 51: }
  5,
  { 52: }
  7,
  15,
  { 53: }
  14,
  { 54: }
  15,
  { 55: }
  10,
  15,
  { 56: }
  13,
  15,
  { 57: }
  8,
  27,
  { 58: }
  11,
  27,
  { 59: }
  27,
  48,
  { 60: }
  17,
  27,
  { 61: }
  18,
  27,
  { 62: }
  19,
  27,
  52,
  { 63: }
  26,
  27,
  { 64: }
  26,
  27,
  { 65: }
  26,
  27,
  { 66: }
  23,
  { 67: }
  24,
  27,
  51,
  { 68: }
  25,
  27,
  { 69: }
  27,
  { 70: }
  27,
  { 71: }
  27,
  40,
  { 72: }
  27,
  41,
  { 73: }
  27,
  42,
  { 74: }
  27,
  43,
  { 75: }
  27,
  44,
  { 76: }
  27,
  45,
  { 77: }
  27,
  46,
  { 78: }
  27,
  47,
  { 79: }
  27,
  49,
  { 80: }
  27,
  50,
  { 81: }
  27,
  53,
  { 82: }
  27,
  54,
  { 83: }
  27,
  55,
  { 84: }
  27,
  56,
  { 85: }
  27,
  57,
  { 86: }
  63,
  { 87: }
  59,
  63,
  { 88: }
  60,
  { 89: }
  61,
  63,
  { 90: }
  62,
  63,
  { 91: }
  63,
  { 92: }
  67,
  { 93: }
  69,
  { 94: }
  68,
  69,
  { 95: }
  69,
  { 96: }
  66,
  { 97: }
  64,
  66,
  { 98: }
  65,
  { 99: }
  66,
  { 100: }
  70,
  { 101: }
  16,
  { 102: }
  36,
  { 103: }
  29,
  { 104: }
  28,
  { 105: }
  { 106: }
  29,
  { 107: }
  30,
  { 108: }
  31,
  { 109: }
  37,
  { 110: }
  32,
  { 111: }
  33,
  { 112: }
  34,
  { 113: }
  35,
  { 114: }
  4,
  { 115: }
  { 116: }
  6,
  { 117: }
  9,
  { 118: }
  17,
  { 119: }
  18,
  { 120: }
  19,
  { 121: }
  19,
  { 122: }
  26,
  { 123: }
  20,
  { 124: }
  26,
  { 125: }
  21,
  { 126: }
  { 127: }
  26,
  { 128: }
  26,
  { 129: }
  59,
  { 130: }
  68,
  { 131: }
  64,
  { 132: }
  28,
  { 133: }
  { 134: }
  { 135: }
  { 136: }
  28,
  { 137: }
  4,
  { 138: }
  22,
  { 139: }
  28
);

yym : array [1..yynmatches] of Integer = (
{ 0: }
{ 1: }
{ 2: }
{ 3: }
{ 4: }
{ 5: }
{ 6: }
{ 7: }
{ 8: }
{ 9: }
{ 10: }
  12,
{ 11: }
  12,
{ 12: }
{ 13: }
{ 14: }
{ 15: }
{ 16: }
{ 17: }
{ 18: }
{ 19: }
{ 20: }
{ 21: }
{ 22: }
  1,
  58,
{ 23: }
  2,
  58,
{ 24: }
  48,
  58,
{ 25: }
  16,
  58,
{ 26: }
  29,
  58,
{ 27: }
  58,
{ 28: }
  30,
  58,
{ 29: }
  56,
  58,
{ 30: }
  50,
  58,
{ 31: }
  45,
  58,
{ 32: }
  46,
  58,
{ 33: }
  38,
{ 34: }
  39,
{ 35: }
  40,
  58,
{ 36: }
  41,
  58,
{ 37: }
  42,
  58,
{ 38: }
  43,
  58,
{ 39: }
  44,
  58,
{ 40: }
  47,
  58,
{ 41: }
  49,
  58,
{ 42: }
  51,
  58,
{ 43: }
  52,
  58,
{ 44: }
  53,
  58,
{ 45: }
  54,
  58,
{ 46: }
  55,
  58,
{ 47: }
  39,
  58,
{ 48: }
  58,
{ 49: }
  3,
  5,
{ 50: }
  5,
{ 51: }
  5,
{ 52: }
  7,
  15,
{ 53: }
  14,
{ 54: }
  15,
{ 55: }
  10,
  15,
{ 56: }
  13,
  15,
{ 57: }
  8,
  27,
{ 58: }
  11,
  27,
{ 59: }
  27,
  48,
{ 60: }
  17,
  27,
{ 61: }
  18,
  27,
{ 62: }
  19,
  27,
  52,
{ 63: }
  26,
  27,
{ 64: }
  26,
  27,
{ 65: }
  26,
  27,
{ 66: }
  23,
{ 67: }
  24,
  27,
  51,
{ 68: }
  25,
  27,
{ 69: }
  27,
{ 70: }
  27,
{ 71: }
  27,
  40,
{ 72: }
  27,
  41,
{ 73: }
  27,
  42,
{ 74: }
  27,
  43,
{ 75: }
  27,
  44,
{ 76: }
  27,
  45,
{ 77: }
  27,
  46,
{ 78: }
  27,
  47,
{ 79: }
  27,
  49,
{ 80: }
  27,
  50,
{ 81: }
  27,
  53,
{ 82: }
  27,
  54,
{ 83: }
  27,
  55,
{ 84: }
  27,
  56,
{ 85: }
  27,
  57,
{ 86: }
  63,
{ 87: }
  59,
  63,
{ 88: }
  60,
{ 89: }
  61,
  63,
{ 90: }
  62,
  63,
{ 91: }
  63,
{ 92: }
  67,
{ 93: }
  69,
{ 94: }
  68,
  69,
{ 95: }
  69,
{ 96: }
  66,
{ 97: }
  64,
  66,
{ 98: }
  65,
{ 99: }
  66,
{ 100: }
  70,
{ 101: }
  16,
{ 102: }
  36,
{ 103: }
  29,
{ 104: }
{ 105: }
{ 106: }
  29,
{ 107: }
  30,
{ 108: }
  31,
{ 109: }
  37,
{ 110: }
  32,
{ 111: }
  33,
{ 112: }
  34,
{ 113: }
  35,
{ 114: }
  4,
{ 115: }
{ 116: }
  6,
{ 117: }
  9,
{ 118: }
  17,
{ 119: }
  18,
{ 120: }
  19,
{ 121: }
  19,
{ 122: }
  26,
{ 123: }
  20,
{ 124: }
  26,
{ 125: }
  21,
{ 126: }
{ 127: }
  26,
{ 128: }
  26,
{ 129: }
  59,
{ 130: }
  68,
{ 131: }
  64,
{ 132: }
  28,
{ 133: }
  28,
{ 134: }
  28,
{ 135: }
{ 136: }
{ 137: }
  4,
{ 138: }
  22,
{ 139: }
  28
);

yyt : array [1..yyntrans] of YYTrec = (
{ 0: }
{ 1: }
{ 2: }
  ( cc: [ #0 ]; s: 33),
  ( cc: [ #1..#9,#11..' ' ]; s: 47),
  ( cc: [ #10 ]; s: 34),
  ( cc: [ '!','"','%','&','?','\','`','|'..#255 ]; s: 48),
  ( cc: [ '#' ]; s: 23),
  ( cc: [ '$' ]; s: 27),
  ( cc: [ '''' ]; s: 22),
  ( cc: [ '(' ]; s: 24),
  ( cc: [ ')' ]; s: 41),
  ( cc: [ '*' ]; s: 37),
  ( cc: [ '+' ]; s: 35),
  ( cc: [ ',' ]; s: 40),
  ( cc: [ '-' ]; s: 36),
  ( cc: [ '.' ]; s: 29),
  ( cc: [ '/' ]; s: 38),
  ( cc: [ '0'..'9' ]; s: 26),
  ( cc: [ ':' ]; s: 30),
  ( cc: [ ';' ]; s: 42),
  ( cc: [ '<' ]; s: 31),
  ( cc: [ '=' ]; s: 39),
  ( cc: [ '>' ]; s: 32),
  ( cc: [ '@' ]; s: 43),
  ( cc: [ 'A'..'Z','_','a'..'z' ]; s: 28),
  ( cc: [ '[' ]; s: 45),
  ( cc: [ ']' ]; s: 46),
  ( cc: [ '^' ]; s: 44),
  ( cc: [ '{' ]; s: 25),
{ 3: }
  ( cc: [ #0 ]; s: 33),
  ( cc: [ #1..#9,#11..' ' ]; s: 47),
  ( cc: [ #10 ]; s: 34),
  ( cc: [ '!','"','%','&','?','\','`','|'..#255 ]; s: 48),
  ( cc: [ '#' ]; s: 23),
  ( cc: [ '$' ]; s: 27),
  ( cc: [ '''' ]; s: 22),
  ( cc: [ '(' ]; s: 24),
  ( cc: [ ')' ]; s: 41),
  ( cc: [ '*' ]; s: 37),
  ( cc: [ '+' ]; s: 35),
  ( cc: [ ',' ]; s: 40),
  ( cc: [ '-' ]; s: 36),
  ( cc: [ '.' ]; s: 29),
  ( cc: [ '/' ]; s: 38),
  ( cc: [ '0'..'9' ]; s: 26),
  ( cc: [ ':' ]; s: 30),
  ( cc: [ ';' ]; s: 42),
  ( cc: [ '<' ]; s: 31),
  ( cc: [ '=' ]; s: 39),
  ( cc: [ '>' ]; s: 32),
  ( cc: [ '@' ]; s: 43),
  ( cc: [ 'A'..'Z','_','a'..'z' ]; s: 28),
  ( cc: [ '[' ]; s: 45),
  ( cc: [ ']' ]; s: 46),
  ( cc: [ '^' ]; s: 44),
  ( cc: [ '{' ]; s: 25),
{ 4: }
  ( cc: [ #1..#9,#11..'"','$'..'&','('..#255 ]; s: 51),
  ( cc: [ '#' ]; s: 50),
  ( cc: [ '''' ]; s: 49),
{ 5: }
  ( cc: [ #1..#9,#11..'"','$'..'&','('..#255 ]; s: 51),
  ( cc: [ '#' ]; s: 50),
  ( cc: [ '''' ]; s: 49),
{ 6: }
  ( cc: [ #1..#9,#11..'&','('..#255 ]; s: 54),
  ( cc: [ #10 ]; s: 53),
  ( cc: [ '''' ]; s: 52),
{ 7: }
  ( cc: [ #1..#9,#11..'&','('..#255 ]; s: 54),
  ( cc: [ #10 ]; s: 53),
  ( cc: [ '''' ]; s: 52),
{ 8: }
  ( cc: [ #1..#9,#11..'&','('..#255 ]; s: 54),
  ( cc: [ #10 ]; s: 53),
  ( cc: [ '''' ]; s: 55),
{ 9: }
  ( cc: [ #1..#9,#11..'&','('..#255 ]; s: 54),
  ( cc: [ #10 ]; s: 53),
  ( cc: [ '''' ]; s: 55),
{ 10: }
  ( cc: [ #1..#9,#11..'!','#'..#255 ]; s: 54),
  ( cc: [ #10 ]; s: 53),
  ( cc: [ '"' ]; s: 56),
{ 11: }
  ( cc: [ #1..#9,#11..'!','#'..#255 ]; s: 54),
  ( cc: [ #10 ]; s: 53),
  ( cc: [ '"' ]; s: 56),
{ 12: }
  ( cc: [ #1..#9,#11..' ' ]; s: 68),
  ( cc: [ #10 ]; s: 66),
  ( cc: [ '!','#','%','?','\','`','|'..#255 ]; s: 70),
  ( cc: [ '"' ]; s: 58),
  ( cc: [ '$' ]; s: 69),
  ( cc: [ '&' ]; s: 85),
  ( cc: [ '''' ]; s: 57),
  ( cc: [ '(' ]; s: 59),
  ( cc: [ ')' ]; s: 79),
  ( cc: [ '*' ]; s: 73),
  ( cc: [ '+' ]; s: 71),
  ( cc: [ ',' ]; s: 78),
  ( cc: [ '-' ]; s: 72),
  ( cc: [ '.' ]; s: 84),
  ( cc: [ '/' ]; s: 74),
  ( cc: [ '0','1' ]; s: 63),
  ( cc: [ '2'..'7' ]; s: 64),
  ( cc: [ '8','9' ]; s: 65),
  ( cc: [ ':' ]; s: 80),
  ( cc: [ ';' ]; s: 67),
  ( cc: [ '<' ]; s: 76),
  ( cc: [ '=' ]; s: 75),
  ( cc: [ '>' ]; s: 77),
  ( cc: [ '@' ]; s: 62),
  ( cc: [ 'A'..'Z','_','a'..'z' ]; s: 61),
  ( cc: [ '[' ]; s: 82),
  ( cc: [ ']' ]; s: 83),
  ( cc: [ '^' ]; s: 81),
  ( cc: [ '{' ]; s: 60),
{ 13: }
  ( cc: [ #1..#9,#11..' ' ]; s: 68),
  ( cc: [ #10 ]; s: 66),
  ( cc: [ '!','#','%','?','\','`','|'..#255 ]; s: 70),
  ( cc: [ '"' ]; s: 58),
  ( cc: [ '$' ]; s: 69),
  ( cc: [ '&' ]; s: 85),
  ( cc: [ '''' ]; s: 57),
  ( cc: [ '(' ]; s: 59),
  ( cc: [ ')' ]; s: 79),
  ( cc: [ '*' ]; s: 73),
  ( cc: [ '+' ]; s: 71),
  ( cc: [ ',' ]; s: 78),
  ( cc: [ '-' ]; s: 72),
  ( cc: [ '.' ]; s: 84),
  ( cc: [ '/' ]; s: 74),
  ( cc: [ '0','1' ]; s: 63),
  ( cc: [ '2'..'7' ]; s: 64),
  ( cc: [ '8','9' ]; s: 65),
  ( cc: [ ':' ]; s: 80),
  ( cc: [ ';' ]; s: 67),
  ( cc: [ '<' ]; s: 76),
  ( cc: [ '=' ]; s: 75),
  ( cc: [ '>' ]; s: 77),
  ( cc: [ '@' ]; s: 62),
  ( cc: [ 'A'..'Z','_','a'..'z' ]; s: 61),
  ( cc: [ '[' ]; s: 82),
  ( cc: [ ']' ]; s: 83),
  ( cc: [ '^' ]; s: 81),
  ( cc: [ '{' ]; s: 60),
{ 14: }
  ( cc: [ #0 ]; s: 88),
  ( cc: [ #1..#9,#11..'!','#'..'&',')'..'z','|'..#255 ]; s: 91),
  ( cc: [ '"' ]; s: 90),
  ( cc: [ '''' ]; s: 89),
  ( cc: [ '(' ]; s: 86),
  ( cc: [ '{' ]; s: 87),
{ 15: }
  ( cc: [ #0 ]; s: 88),
  ( cc: [ #1..#9,#11..'!','#'..'&',')'..'z','|'..#255 ]; s: 91),
  ( cc: [ '"' ]; s: 90),
  ( cc: [ '''' ]; s: 89),
  ( cc: [ '(' ]; s: 86),
  ( cc: [ '{' ]; s: 87),
{ 16: }
  ( cc: [ #0 ]; s: 92),
  ( cc: [ #1..#9,#11..')','+'..'|','~'..#255 ]; s: 95),
  ( cc: [ '*' ]; s: 93),
  ( cc: [ '}' ]; s: 94),
{ 17: }
  ( cc: [ #0 ]; s: 92),
  ( cc: [ #1..#9,#11..')','+'..'|','~'..#255 ]; s: 95),
  ( cc: [ '*' ]; s: 93),
  ( cc: [ '}' ]; s: 94),
{ 18: }
  ( cc: [ #0 ]; s: 98),
  ( cc: [ #1..#9,#11..')','+'..'|','~'..#255 ]; s: 99),
  ( cc: [ '*' ]; s: 96),
  ( cc: [ '}' ]; s: 97),
{ 19: }
  ( cc: [ #0 ]; s: 98),
  ( cc: [ #1..#9,#11..')','+'..'|','~'..#255 ]; s: 99),
  ( cc: [ '*' ]; s: 96),
  ( cc: [ '}' ]; s: 97),
{ 20: }
  ( cc: [ #1..#9,#11..#255 ]; s: 100),
{ 21: }
  ( cc: [ #1..#9,#11..#255 ]; s: 100),
{ 22: }
{ 23: }
{ 24: }
  ( cc: [ '*' ]; s: 101),
  ( cc: [ '.' ]; s: 102),
{ 25: }
{ 26: }
  ( cc: [ '.' ]; s: 104),
  ( cc: [ '0'..'9' ]; s: 103),
  ( cc: [ 'E','e' ]; s: 105),
{ 27: }
  ( cc: [ '0'..'9','A'..'F','a'..'f' ]; s: 106),
{ 28: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'z' ]; s: 107),
{ 29: }
  ( cc: [ ')' ]; s: 109),
  ( cc: [ '.' ]; s: 108),
{ 30: }
  ( cc: [ '=' ]; s: 110),
{ 31: }
  ( cc: [ '=' ]; s: 111),
  ( cc: [ '>' ]; s: 112),
{ 32: }
  ( cc: [ '=' ]; s: 113),
{ 33: }
{ 34: }
{ 35: }
{ 36: }
{ 37: }
{ 38: }
{ 39: }
{ 40: }
{ 41: }
{ 42: }
{ 43: }
{ 44: }
{ 45: }
{ 46: }
{ 47: }
{ 48: }
{ 49: }
{ 50: }
  ( cc: [ '$' ]; s: 115),
  ( cc: [ '0'..'9' ]; s: 114),
{ 51: }
{ 52: }
  ( cc: [ '''' ]; s: 116),
{ 53: }
{ 54: }
{ 55: }
  ( cc: [ '''' ]; s: 117),
{ 56: }
{ 57: }
{ 58: }
{ 59: }
  ( cc: [ '*' ]; s: 118),
{ 60: }
{ 61: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'z' ]; s: 119),
{ 62: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'z' ]; s: 121),
  ( cc: [ '@' ]; s: 120),
{ 63: }
  ( cc: [ '0','1' ]; s: 122),
  ( cc: [ '2'..'7' ]; s: 124),
  ( cc: [ '8','9' ]; s: 127),
  ( cc: [ 'A','C'..'F','a','c'..'f' ]; s: 126),
  ( cc: [ 'B','b' ]; s: 123),
  ( cc: [ 'O','o' ]; s: 125),
{ 64: }
  ( cc: [ '0'..'7' ]; s: 124),
  ( cc: [ '8','9' ]; s: 127),
  ( cc: [ 'A'..'F','a'..'f' ]; s: 126),
  ( cc: [ 'O','o' ]; s: 125),
{ 65: }
  ( cc: [ '0'..'9' ]; s: 127),
  ( cc: [ 'A'..'F','a'..'f' ]; s: 126),
{ 66: }
{ 67: }
{ 68: }
{ 69: }
  ( cc: [ '0'..'9','A'..'F','a'..'f' ]; s: 128),
{ 70: }
{ 71: }
{ 72: }
{ 73: }
{ 74: }
{ 75: }
{ 76: }
{ 77: }
{ 78: }
{ 79: }
{ 80: }
{ 81: }
{ 82: }
{ 83: }
{ 84: }
{ 85: }
{ 86: }
  ( cc: [ '*' ]; s: 129),
{ 87: }
{ 88: }
{ 89: }
{ 90: }
{ 91: }
{ 92: }
{ 93: }
  ( cc: [ ')' ]; s: 130),
{ 94: }
{ 95: }
{ 96: }
  ( cc: [ ')' ]; s: 131),
{ 97: }
{ 98: }
{ 99: }
{ 100: }
{ 101: }
{ 102: }
{ 103: }
  ( cc: [ '.' ]; s: 104),
  ( cc: [ '0'..'9' ]; s: 103),
  ( cc: [ 'E','e' ]; s: 105),
{ 104: }
  ( cc: [ #1..'-','/',':'..'D','F'..'d','f'..#255 ]; s: 134),
  ( cc: [ '0'..'9' ]; s: 132),
  ( cc: [ 'E','e' ]; s: 133),
{ 105: }
  ( cc: [ '+','-' ]; s: 135),
  ( cc: [ '0'..'9' ]; s: 136),
{ 106: }
  ( cc: [ '0'..'9','A'..'F','a'..'f' ]; s: 106),
{ 107: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'z' ]; s: 107),
{ 108: }
{ 109: }
{ 110: }
{ 111: }
{ 112: }
{ 113: }
{ 114: }
  ( cc: [ '0'..'9' ]; s: 114),
{ 115: }
  ( cc: [ '0'..'9','A'..'F','a'..'f' ]; s: 137),
{ 116: }
{ 117: }
{ 118: }
{ 119: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'z' ]; s: 119),
{ 120: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'z' ]; s: 121),
  ( cc: [ '@' ]; s: 120),
{ 121: }
  ( cc: [ '0'..'9','@'..'Z','_','a'..'z' ]; s: 121),
{ 122: }
  ( cc: [ '0','1' ]; s: 122),
  ( cc: [ '2'..'7' ]; s: 124),
  ( cc: [ '8','9' ]; s: 127),
  ( cc: [ 'A','C'..'F','a','c'..'f' ]; s: 126),
  ( cc: [ 'B','b' ]; s: 123),
  ( cc: [ 'H','h' ]; s: 138),
  ( cc: [ 'O','o' ]; s: 125),
{ 123: }
  ( cc: [ '0'..'9','A'..'F','a'..'f' ]; s: 126),
  ( cc: [ 'H','h' ]; s: 138),
{ 124: }
  ( cc: [ '0'..'7' ]; s: 124),
  ( cc: [ '8','9' ]; s: 127),
  ( cc: [ 'A'..'F','a'..'f' ]; s: 126),
  ( cc: [ 'H','h' ]; s: 138),
  ( cc: [ 'O','o' ]; s: 125),
{ 125: }
{ 126: }
  ( cc: [ '0'..'9','A'..'F','a'..'f' ]; s: 126),
  ( cc: [ 'H','h' ]; s: 138),
{ 127: }
  ( cc: [ '0'..'9' ]; s: 127),
  ( cc: [ 'A'..'F','a'..'f' ]; s: 126),
  ( cc: [ 'H','h' ]; s: 138),
{ 128: }
  ( cc: [ '0'..'9','A'..'F','a'..'f' ]; s: 128),
{ 129: }
{ 130: }
{ 131: }
{ 132: }
  ( cc: [ #1..'-','/',':'..'D','F'..'d','f'..#255 ]; s: 134),
  ( cc: [ '0'..'9' ]; s: 132),
  ( cc: [ 'E','e' ]; s: 133),
{ 133: }
  ( cc: [ '+','-' ]; s: 135),
  ( cc: [ '0'..'9' ]; s: 136),
{ 134: }
{ 135: }
  ( cc: [ '0'..'9' ]; s: 136),
{ 136: }
  ( cc: [ #1..'-','/',':'..#255 ]; s: 134),
  ( cc: [ '0'..'9' ]; s: 139),
{ 137: }
  ( cc: [ '0'..'9','A'..'F','a'..'f' ]; s: 137),
{ 138: }
{ 139: }
  ( cc: [ #1..'-','/',':'..#255 ]; s: 134),
  ( cc: [ '0'..'9' ]; s: 139)
);

yykl : array [0..yynstates-1] of Integer = (
{ 0: } 1,
{ 1: } 1,
{ 2: } 1,
{ 3: } 1,
{ 4: } 1,
{ 5: } 1,
{ 6: } 1,
{ 7: } 1,
{ 8: } 1,
{ 9: } 1,
{ 10: } 1,
{ 11: } 2,
{ 12: } 3,
{ 13: } 3,
{ 14: } 3,
{ 15: } 3,
{ 16: } 3,
{ 17: } 3,
{ 18: } 3,
{ 19: } 3,
{ 20: } 3,
{ 21: } 3,
{ 22: } 3,
{ 23: } 5,
{ 24: } 7,
{ 25: } 9,
{ 26: } 11,
{ 27: } 13,
{ 28: } 14,
{ 29: } 16,
{ 30: } 18,
{ 31: } 20,
{ 32: } 22,
{ 33: } 24,
{ 34: } 25,
{ 35: } 26,
{ 36: } 28,
{ 37: } 30,
{ 38: } 32,
{ 39: } 34,
{ 40: } 36,
{ 41: } 38,
{ 42: } 40,
{ 43: } 42,
{ 44: } 44,
{ 45: } 46,
{ 46: } 48,
{ 47: } 50,
{ 48: } 52,
{ 49: } 53,
{ 50: } 55,
{ 51: } 56,
{ 52: } 57,
{ 53: } 59,
{ 54: } 60,
{ 55: } 61,
{ 56: } 63,
{ 57: } 65,
{ 58: } 67,
{ 59: } 69,
{ 60: } 71,
{ 61: } 73,
{ 62: } 75,
{ 63: } 78,
{ 64: } 80,
{ 65: } 82,
{ 66: } 84,
{ 67: } 85,
{ 68: } 88,
{ 69: } 90,
{ 70: } 91,
{ 71: } 92,
{ 72: } 94,
{ 73: } 96,
{ 74: } 98,
{ 75: } 100,
{ 76: } 102,
{ 77: } 104,
{ 78: } 106,
{ 79: } 108,
{ 80: } 110,
{ 81: } 112,
{ 82: } 114,
{ 83: } 116,
{ 84: } 118,
{ 85: } 120,
{ 86: } 122,
{ 87: } 123,
{ 88: } 125,
{ 89: } 126,
{ 90: } 128,
{ 91: } 130,
{ 92: } 131,
{ 93: } 132,
{ 94: } 133,
{ 95: } 135,
{ 96: } 136,
{ 97: } 137,
{ 98: } 139,
{ 99: } 140,
{ 100: } 141,
{ 101: } 142,
{ 102: } 143,
{ 103: } 144,
{ 104: } 145,
{ 105: } 146,
{ 106: } 146,
{ 107: } 147,
{ 108: } 148,
{ 109: } 149,
{ 110: } 150,
{ 111: } 151,
{ 112: } 152,
{ 113: } 153,
{ 114: } 154,
{ 115: } 155,
{ 116: } 155,
{ 117: } 156,
{ 118: } 157,
{ 119: } 158,
{ 120: } 159,
{ 121: } 160,
{ 122: } 161,
{ 123: } 162,
{ 124: } 163,
{ 125: } 164,
{ 126: } 165,
{ 127: } 165,
{ 128: } 166,
{ 129: } 167,
{ 130: } 168,
{ 131: } 169,
{ 132: } 170,
{ 133: } 171,
{ 134: } 171,
{ 135: } 171,
{ 136: } 171,
{ 137: } 172,
{ 138: } 173,
{ 139: } 174
);

yykh : array [0..yynstates-1] of Integer = (
{ 0: } 0,
{ 1: } 0,
{ 2: } 0,
{ 3: } 0,
{ 4: } 0,
{ 5: } 0,
{ 6: } 0,
{ 7: } 0,
{ 8: } 0,
{ 9: } 0,
{ 10: } 1,
{ 11: } 2,
{ 12: } 2,
{ 13: } 2,
{ 14: } 2,
{ 15: } 2,
{ 16: } 2,
{ 17: } 2,
{ 18: } 2,
{ 19: } 2,
{ 20: } 2,
{ 21: } 2,
{ 22: } 4,
{ 23: } 6,
{ 24: } 8,
{ 25: } 10,
{ 26: } 12,
{ 27: } 13,
{ 28: } 15,
{ 29: } 17,
{ 30: } 19,
{ 31: } 21,
{ 32: } 23,
{ 33: } 24,
{ 34: } 25,
{ 35: } 27,
{ 36: } 29,
{ 37: } 31,
{ 38: } 33,
{ 39: } 35,
{ 40: } 37,
{ 41: } 39,
{ 42: } 41,
{ 43: } 43,
{ 44: } 45,
{ 45: } 47,
{ 46: } 49,
{ 47: } 51,
{ 48: } 52,
{ 49: } 54,
{ 50: } 55,
{ 51: } 56,
{ 52: } 58,
{ 53: } 59,
{ 54: } 60,
{ 55: } 62,
{ 56: } 64,
{ 57: } 66,
{ 58: } 68,
{ 59: } 70,
{ 60: } 72,
{ 61: } 74,
{ 62: } 77,
{ 63: } 79,
{ 64: } 81,
{ 65: } 83,
{ 66: } 84,
{ 67: } 87,
{ 68: } 89,
{ 69: } 90,
{ 70: } 91,
{ 71: } 93,
{ 72: } 95,
{ 73: } 97,
{ 74: } 99,
{ 75: } 101,
{ 76: } 103,
{ 77: } 105,
{ 78: } 107,
{ 79: } 109,
{ 80: } 111,
{ 81: } 113,
{ 82: } 115,
{ 83: } 117,
{ 84: } 119,
{ 85: } 121,
{ 86: } 122,
{ 87: } 124,
{ 88: } 125,
{ 89: } 127,
{ 90: } 129,
{ 91: } 130,
{ 92: } 131,
{ 93: } 132,
{ 94: } 134,
{ 95: } 135,
{ 96: } 136,
{ 97: } 138,
{ 98: } 139,
{ 99: } 140,
{ 100: } 141,
{ 101: } 142,
{ 102: } 143,
{ 103: } 144,
{ 104: } 145,
{ 105: } 145,
{ 106: } 146,
{ 107: } 147,
{ 108: } 148,
{ 109: } 149,
{ 110: } 150,
{ 111: } 151,
{ 112: } 152,
{ 113: } 153,
{ 114: } 154,
{ 115: } 154,
{ 116: } 155,
{ 117: } 156,
{ 118: } 157,
{ 119: } 158,
{ 120: } 159,
{ 121: } 160,
{ 122: } 161,
{ 123: } 162,
{ 124: } 163,
{ 125: } 164,
{ 126: } 164,
{ 127: } 165,
{ 128: } 166,
{ 129: } 167,
{ 130: } 168,
{ 131: } 169,
{ 132: } 170,
{ 133: } 170,
{ 134: } 170,
{ 135: } 170,
{ 136: } 171,
{ 137: } 172,
{ 138: } 173,
{ 139: } 174
);

yyml : array [0..yynstates-1] of Integer = (
{ 0: } 1,
{ 1: } 1,
{ 2: } 1,
{ 3: } 1,
{ 4: } 1,
{ 5: } 1,
{ 6: } 1,
{ 7: } 1,
{ 8: } 1,
{ 9: } 1,
{ 10: } 1,
{ 11: } 2,
{ 12: } 3,
{ 13: } 3,
{ 14: } 3,
{ 15: } 3,
{ 16: } 3,
{ 17: } 3,
{ 18: } 3,
{ 19: } 3,
{ 20: } 3,
{ 21: } 3,
{ 22: } 3,
{ 23: } 5,
{ 24: } 7,
{ 25: } 9,
{ 26: } 11,
{ 27: } 13,
{ 28: } 14,
{ 29: } 16,
{ 30: } 18,
{ 31: } 20,
{ 32: } 22,
{ 33: } 24,
{ 34: } 25,
{ 35: } 26,
{ 36: } 28,
{ 37: } 30,
{ 38: } 32,
{ 39: } 34,
{ 40: } 36,
{ 41: } 38,
{ 42: } 40,
{ 43: } 42,
{ 44: } 44,
{ 45: } 46,
{ 46: } 48,
{ 47: } 50,
{ 48: } 52,
{ 49: } 53,
{ 50: } 55,
{ 51: } 56,
{ 52: } 57,
{ 53: } 59,
{ 54: } 60,
{ 55: } 61,
{ 56: } 63,
{ 57: } 65,
{ 58: } 67,
{ 59: } 69,
{ 60: } 71,
{ 61: } 73,
{ 62: } 75,
{ 63: } 78,
{ 64: } 80,
{ 65: } 82,
{ 66: } 84,
{ 67: } 85,
{ 68: } 88,
{ 69: } 90,
{ 70: } 91,
{ 71: } 92,
{ 72: } 94,
{ 73: } 96,
{ 74: } 98,
{ 75: } 100,
{ 76: } 102,
{ 77: } 104,
{ 78: } 106,
{ 79: } 108,
{ 80: } 110,
{ 81: } 112,
{ 82: } 114,
{ 83: } 116,
{ 84: } 118,
{ 85: } 120,
{ 86: } 122,
{ 87: } 123,
{ 88: } 125,
{ 89: } 126,
{ 90: } 128,
{ 91: } 130,
{ 92: } 131,
{ 93: } 132,
{ 94: } 133,
{ 95: } 135,
{ 96: } 136,
{ 97: } 137,
{ 98: } 139,
{ 99: } 140,
{ 100: } 141,
{ 101: } 142,
{ 102: } 143,
{ 103: } 144,
{ 104: } 145,
{ 105: } 145,
{ 106: } 145,
{ 107: } 146,
{ 108: } 147,
{ 109: } 148,
{ 110: } 149,
{ 111: } 150,
{ 112: } 151,
{ 113: } 152,
{ 114: } 153,
{ 115: } 154,
{ 116: } 154,
{ 117: } 155,
{ 118: } 156,
{ 119: } 157,
{ 120: } 158,
{ 121: } 159,
{ 122: } 160,
{ 123: } 161,
{ 124: } 162,
{ 125: } 163,
{ 126: } 164,
{ 127: } 164,
{ 128: } 165,
{ 129: } 166,
{ 130: } 167,
{ 131: } 168,
{ 132: } 169,
{ 133: } 170,
{ 134: } 171,
{ 135: } 172,
{ 136: } 172,
{ 137: } 172,
{ 138: } 173,
{ 139: } 174
);

yymh : array [0..yynstates-1] of Integer = (
{ 0: } 0,
{ 1: } 0,
{ 2: } 0,
{ 3: } 0,
{ 4: } 0,
{ 5: } 0,
{ 6: } 0,
{ 7: } 0,
{ 8: } 0,
{ 9: } 0,
{ 10: } 1,
{ 11: } 2,
{ 12: } 2,
{ 13: } 2,
{ 14: } 2,
{ 15: } 2,
{ 16: } 2,
{ 17: } 2,
{ 18: } 2,
{ 19: } 2,
{ 20: } 2,
{ 21: } 2,
{ 22: } 4,
{ 23: } 6,
{ 24: } 8,
{ 25: } 10,
{ 26: } 12,
{ 27: } 13,
{ 28: } 15,
{ 29: } 17,
{ 30: } 19,
{ 31: } 21,
{ 32: } 23,
{ 33: } 24,
{ 34: } 25,
{ 35: } 27,
{ 36: } 29,
{ 37: } 31,
{ 38: } 33,
{ 39: } 35,
{ 40: } 37,
{ 41: } 39,
{ 42: } 41,
{ 43: } 43,
{ 44: } 45,
{ 45: } 47,
{ 46: } 49,
{ 47: } 51,
{ 48: } 52,
{ 49: } 54,
{ 50: } 55,
{ 51: } 56,
{ 52: } 58,
{ 53: } 59,
{ 54: } 60,
{ 55: } 62,
{ 56: } 64,
{ 57: } 66,
{ 58: } 68,
{ 59: } 70,
{ 60: } 72,
{ 61: } 74,
{ 62: } 77,
{ 63: } 79,
{ 64: } 81,
{ 65: } 83,
{ 66: } 84,
{ 67: } 87,
{ 68: } 89,
{ 69: } 90,
{ 70: } 91,
{ 71: } 93,
{ 72: } 95,
{ 73: } 97,
{ 74: } 99,
{ 75: } 101,
{ 76: } 103,
{ 77: } 105,
{ 78: } 107,
{ 79: } 109,
{ 80: } 111,
{ 81: } 113,
{ 82: } 115,
{ 83: } 117,
{ 84: } 119,
{ 85: } 121,
{ 86: } 122,
{ 87: } 124,
{ 88: } 125,
{ 89: } 127,
{ 90: } 129,
{ 91: } 130,
{ 92: } 131,
{ 93: } 132,
{ 94: } 134,
{ 95: } 135,
{ 96: } 136,
{ 97: } 138,
{ 98: } 139,
{ 99: } 140,
{ 100: } 141,
{ 101: } 142,
{ 102: } 143,
{ 103: } 144,
{ 104: } 144,
{ 105: } 144,
{ 106: } 145,
{ 107: } 146,
{ 108: } 147,
{ 109: } 148,
{ 110: } 149,
{ 111: } 150,
{ 112: } 151,
{ 113: } 152,
{ 114: } 153,
{ 115: } 153,
{ 116: } 154,
{ 117: } 155,
{ 118: } 156,
{ 119: } 157,
{ 120: } 158,
{ 121: } 159,
{ 122: } 160,
{ 123: } 161,
{ 124: } 162,
{ 125: } 163,
{ 126: } 163,
{ 127: } 164,
{ 128: } 165,
{ 129: } 166,
{ 130: } 167,
{ 131: } 168,
{ 132: } 169,
{ 133: } 170,
{ 134: } 171,
{ 135: } 171,
{ 136: } 171,
{ 137: } 172,
{ 138: } 173,
{ 139: } 174
);

yytl : array [0..yynstates-1] of Integer = (
{ 0: } 1,
{ 1: } 1,
{ 2: } 1,
{ 3: } 28,
{ 4: } 55,
{ 5: } 58,
{ 6: } 61,
{ 7: } 64,
{ 8: } 67,
{ 9: } 70,
{ 10: } 73,
{ 11: } 76,
{ 12: } 79,
{ 13: } 108,
{ 14: } 137,
{ 15: } 143,
{ 16: } 149,
{ 17: } 153,
{ 18: } 157,
{ 19: } 161,
{ 20: } 165,
{ 21: } 166,
{ 22: } 167,
{ 23: } 167,
{ 24: } 167,
{ 25: } 169,
{ 26: } 169,
{ 27: } 172,
{ 28: } 173,
{ 29: } 174,
{ 30: } 176,
{ 31: } 177,
{ 32: } 179,
{ 33: } 180,
{ 34: } 180,
{ 35: } 180,
{ 36: } 180,
{ 37: } 180,
{ 38: } 180,
{ 39: } 180,
{ 40: } 180,
{ 41: } 180,
{ 42: } 180,
{ 43: } 180,
{ 44: } 180,
{ 45: } 180,
{ 46: } 180,
{ 47: } 180,
{ 48: } 180,
{ 49: } 180,
{ 50: } 180,
{ 51: } 182,
{ 52: } 182,
{ 53: } 183,
{ 54: } 183,
{ 55: } 183,
{ 56: } 184,
{ 57: } 184,
{ 58: } 184,
{ 59: } 184,
{ 60: } 185,
{ 61: } 185,
{ 62: } 186,
{ 63: } 188,
{ 64: } 194,
{ 65: } 198,
{ 66: } 200,
{ 67: } 200,
{ 68: } 200,
{ 69: } 200,
{ 70: } 201,
{ 71: } 201,
{ 72: } 201,
{ 73: } 201,
{ 74: } 201,
{ 75: } 201,
{ 76: } 201,
{ 77: } 201,
{ 78: } 201,
{ 79: } 201,
{ 80: } 201,
{ 81: } 201,
{ 82: } 201,
{ 83: } 201,
{ 84: } 201,
{ 85: } 201,
{ 86: } 201,
{ 87: } 202,
{ 88: } 202,
{ 89: } 202,
{ 90: } 202,
{ 91: } 202,
{ 92: } 202,
{ 93: } 202,
{ 94: } 203,
{ 95: } 203,
{ 96: } 203,
{ 97: } 204,
{ 98: } 204,
{ 99: } 204,
{ 100: } 204,
{ 101: } 204,
{ 102: } 204,
{ 103: } 204,
{ 104: } 207,
{ 105: } 210,
{ 106: } 212,
{ 107: } 213,
{ 108: } 214,
{ 109: } 214,
{ 110: } 214,
{ 111: } 214,
{ 112: } 214,
{ 113: } 214,
{ 114: } 214,
{ 115: } 215,
{ 116: } 216,
{ 117: } 216,
{ 118: } 216,
{ 119: } 216,
{ 120: } 217,
{ 121: } 219,
{ 122: } 220,
{ 123: } 227,
{ 124: } 229,
{ 125: } 234,
{ 126: } 234,
{ 127: } 236,
{ 128: } 239,
{ 129: } 240,
{ 130: } 240,
{ 131: } 240,
{ 132: } 240,
{ 133: } 243,
{ 134: } 245,
{ 135: } 245,
{ 136: } 246,
{ 137: } 248,
{ 138: } 249,
{ 139: } 249
);

yyth : array [0..yynstates-1] of Integer = (
{ 0: } 0,
{ 1: } 0,
{ 2: } 27,
{ 3: } 54,
{ 4: } 57,
{ 5: } 60,
{ 6: } 63,
{ 7: } 66,
{ 8: } 69,
{ 9: } 72,
{ 10: } 75,
{ 11: } 78,
{ 12: } 107,
{ 13: } 136,
{ 14: } 142,
{ 15: } 148,
{ 16: } 152,
{ 17: } 156,
{ 18: } 160,
{ 19: } 164,
{ 20: } 165,
{ 21: } 166,
{ 22: } 166,
{ 23: } 166,
{ 24: } 168,
{ 25: } 168,
{ 26: } 171,
{ 27: } 172,
{ 28: } 173,
{ 29: } 175,
{ 30: } 176,
{ 31: } 178,
{ 32: } 179,
{ 33: } 179,
{ 34: } 179,
{ 35: } 179,
{ 36: } 179,
{ 37: } 179,
{ 38: } 179,
{ 39: } 179,
{ 40: } 179,
{ 41: } 179,
{ 42: } 179,
{ 43: } 179,
{ 44: } 179,
{ 45: } 179,
{ 46: } 179,
{ 47: } 179,
{ 48: } 179,
{ 49: } 179,
{ 50: } 181,
{ 51: } 181,
{ 52: } 182,
{ 53: } 182,
{ 54: } 182,
{ 55: } 183,
{ 56: } 183,
{ 57: } 183,
{ 58: } 183,
{ 59: } 184,
{ 60: } 184,
{ 61: } 185,
{ 62: } 187,
{ 63: } 193,
{ 64: } 197,
{ 65: } 199,
{ 66: } 199,
{ 67: } 199,
{ 68: } 199,
{ 69: } 200,
{ 70: } 200,
{ 71: } 200,
{ 72: } 200,
{ 73: } 200,
{ 74: } 200,
{ 75: } 200,
{ 76: } 200,
{ 77: } 200,
{ 78: } 200,
{ 79: } 200,
{ 80: } 200,
{ 81: } 200,
{ 82: } 200,
{ 83: } 200,
{ 84: } 200,
{ 85: } 200,
{ 86: } 201,
{ 87: } 201,
{ 88: } 201,
{ 89: } 201,
{ 90: } 201,
{ 91: } 201,
{ 92: } 201,
{ 93: } 202,
{ 94: } 202,
{ 95: } 202,
{ 96: } 203,
{ 97: } 203,
{ 98: } 203,
{ 99: } 203,
{ 100: } 203,
{ 101: } 203,
{ 102: } 203,
{ 103: } 206,
{ 104: } 209,
{ 105: } 211,
{ 106: } 212,
{ 107: } 213,
{ 108: } 213,
{ 109: } 213,
{ 110: } 213,
{ 111: } 213,
{ 112: } 213,
{ 113: } 213,
{ 114: } 214,
{ 115: } 215,
{ 116: } 215,
{ 117: } 215,
{ 118: } 215,
{ 119: } 216,
{ 120: } 218,
{ 121: } 219,
{ 122: } 226,
{ 123: } 228,
{ 124: } 233,
{ 125: } 233,
{ 126: } 235,
{ 127: } 238,
{ 128: } 239,
{ 129: } 239,
{ 130: } 239,
{ 131: } 239,
{ 132: } 242,
{ 133: } 244,
{ 134: } 244,
{ 135: } 245,
{ 136: } 247,
{ 137: } 248,
{ 138: } 248,
{ 139: } 250
);


function yylex : Integer;

var yyn : Integer;

label start, scan, action;

begin

start:

  (* initialize: *)

  yynew;

scan:

  (* mark positions and matches: *)

  for yyn := yykl[yystate] to     yykh[yystate] do yymark(yyk[yyn]);
  for yyn := yymh[yystate] downto yyml[yystate] do yymatch(yym[yyn]);

  if yytl[yystate]>yyth[yystate] then goto action; (* dead state *)

  (* get next character: *)

  yyscan;

  (* determine action: *)

  yyn := yytl[yystate];
  while (yyn<=yyth[yystate]) and not (yyactchar in yyt[yyn].cc) do inc(yyn);
  if yyn>yyth[yystate] then goto action;
    (* no transition on yyactchar in this state *)

  (* switch to new state: *)

  yystate := yyt[yyn].s;

  goto scan;

action:

  (* execute action: *)

  if yyfind(yyrule) then
    begin
      yyaction(yyrule);
      {$ifdef debug}
      write( yylineno,':',yycolno,'|',yyretval,'|',yyline,'|',yytext );
      {$endif}
      if yyreject then goto action;
    end
  else if not yydefault and yywrap then
    begin
      yyclear;
      return(0);
    end;

  if not yydone then goto start;

  yylex := yyretval;

end(*yylex*);

initialization
  start( Normal );
end.

