unit Errors;

interface
uses SysUtils;

type
   TErrorSeverity = ( es_Message, es_Information, es_Hint, es_Warning, es_Error, es_Fatal );
   ECompilerError = class( Exception )
     protected
       ModuleCode : word;
       Severity : TErrorSeverity;
       ErrorCode : word;
     public
       constructor CreateError( const FileName: PString; LineNo, ColNo : longint;
                                ErrorModule : word; ErrorS : word;
                                const Args : array of const );
   end;

{$I ErrorMod.inc}
{$I Errors.inc}
{$R Errors.res}
implementation
uses Dialogs,Forms;

const ErrorMessagePrefix = '%s(%d,%d):%.4s.%.2d.%.4d:';

constructor ECompilerError.CreateError( const FileName: PString; LineNo, ColNo : longint;
                                        ErrorModule : word; ErrorS : word;
                                        const Args: array of const );
var AHelpContext: longint;
    s : string;
begin
  ModuleCode := ErrorModule;
  Severity := TErrorSeverity( ErrorS div 10000 );
  ErrorCode := ErrorS mod 10000;
  AHelpContext := ErrorS;
  s := Format( ErrorMessagePrefix, [ FileName^, LineNo, ColNo, LoadStr( ModuleCode ),
                                     ord( Severity ), ErrorCode ] ) +
          LoadStr( ErrorS ) + '%s';
  inherited CreateFmtHelp( s, Args,  AHelpContext );
end;

initialization
end.
